/* CAMTYPE.C   Camera Type definition Utility for PCprocess Modules MGVS, MRGB
                    and PCimage Module MGVSi

    uf 14.03.1995
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>


#define MV_DLL

#include "include\mv.h"
#include "dlldrv\camtype.h"

#define UNUSED(v)  if(v) v=v

#define DIG_MGVS    1
#define DIG_MRGB    2
#define DIG_MGVSi   3
#define DIG_PCISG   4
#define DIG_PCISC   5
#define DIG_PCISRGB 6
#define DIG_PCISDIG 7


static LPCAMTYPE_T pActCamType      = NULL;
static LPCAMTYPE_T pFirstCamType    = NULL;
static WORD        NumCamTypes      = 0;
static WORD        MaxNumCamTypes   = 0;
static char FAR*   slVideoModes[]   = CAM_VIDEO_MODES;
static char FAR*   slInterlModes[]  = CAM_INTERL_MODES;
static char FAR*   slAcquireModes[] = CAM_ACQUIRE_MODES;
static char FAR*   slStartModes[]   = CAM_START_MODES;
static char FAR*   slFilterModes[]  = CAM_FILTER;
static char FAR*   slParModes[]     = CAM_UNIT_MODES;
static char FAR*   slClockModes[]   = CAM_CLOCK_MODES;
static char FAR*   slInvertModes[]  = CAM_INVERT_MODES;

CAMTYPE_T FAR *pdefault_camdef = default_camdef ;


/* ------------------------------ GetStrValue --------------------------- */

static WORD GetStrValue (DEV_T FAR* dev,
                         char FAR* str,
                         WORD default_value,
                         char FAR* FAR* strlist)
{
    char FAR* FAR* p = strlist;
    WORD result = 0;

    strupr ((char*) str);

    while (*p != NULL)
    {
        if (strcmp (*p, str) == 0)
      break;

        result++;
        p++;
    }

    if (*p == NULL)
    {
        //SetErrStr (dev, ERR_ILLEGAL_PARAM, str);
        SetErr(dev, ERR_ILLEGAL_PARAM);
        result = default_value;
    }

    return result;
}


/* ------------------------------ mvInitCamTypeDefinition --------------- */

CAMDEFAPI(WORD) mvInitCamTypeDefinitions (WORD MaxCameraDefs)
{
    mvAddUserFunction ("SelCamera"              , (USER_FUNC_T) mvSelCamera);
    mvAddUserFunction ("DefCamType"             , (USER_FUNC_T) mvDefCamType);
    mvAddUserFunction ("DefCamAcquireSetup"     , (USER_FUNC_T) mvDefCamAcquireSetup);
    mvAddUserFunction ("DefCamGenerateVSync"    , (USER_FUNC_T) mvDefCamGenerateVSync);
    mvAddUserFunction ("DefCamAnalogParam"      , (USER_FUNC_T) mvDefCamAnalogParam);
    mvAddUserFunction ("DefHorizontalUnit"      , (USER_FUNC_T) mvDefHorizontalUnit);
    mvAddUserFunction ("DefVerticalUnit"        , (USER_FUNC_T) mvDefVerticalUnit);
    mvAddUserFunction ("DefCamHorizontalAcquire", (USER_FUNC_T) mvDefCamHorizontalAcquire);
    mvAddUserFunction ("DefCamVerticalAcquire"  , (USER_FUNC_T) mvDefCamVerticalAcquire);
    mvAddUserFunction ("DefCamClamp"            , (USER_FUNC_T) mvDefCamClamp);
    mvAddUserFunction ("DefCamZero"             , (USER_FUNC_T) mvDefCamZero);
    mvAddUserFunction ("DefCamFieldGate"        , (USER_FUNC_T) mvDefCamFieldGate);
    mvAddUserFunction ("DefCamClamp"            , (USER_FUNC_T) mvDefCamClamp);
    mvAddUserFunction ("DefChannelGain"         , (USER_FUNC_T) mvDefChannelGain);
    mvAddUserFunction ("DefChannelOffset"       , (USER_FUNC_T) mvDefChannelOffset);
    mvAddUserFunction ("DefChannelReference"    , (USER_FUNC_T) mvDefChannelReference);

    MaxNumCamTypes = MaxCameraDefs;

    if ((pFirstCamType = (LPCAMTYPE_T) calloc (MaxCameraDefs, sizeof (CAMTYPE_T))) == NULL)
    {
        return ERR_NO_MEM;
    }

    return NO_ERR;
}


/* ------------------------------ mvFreeCamTypeDefinitions -------------- */

CAMDEFAPI(VOID) mvFreeCamTypeDefinitions ()
{
   if (pFirstCamType != NULL)
     free (pFirstCamType);

   pActCamType   = NULL;
   pFirstCamType = NULL;
   NumCamTypes   = 0;
}


/* ------------------------------ mvDefCamType -------------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamType (DEV_T FAR* dev,
                           char FAR* Name,
                           char FAR* VideoMode,
                           char FAR* Interlaced,
                           WORD VFreq,
                           WORD HFreq,
                           WORD ClockFreq,
                           char FAR* ClockMode)
{
   LPCAMTYPE_T pCam;
   WORD nVideoMode ;

   if (pFirstCamType == NULL)
   {
     SetErr (dev, ERR_NO_MEM);
     return;
   }

   pCam = pFirstCamType + NumCamTypes;

   if (NumCamTypes < MaxNumCamTypes)
     NumCamTypes ++;
   else
     return;

   nVideoMode  = GetStrValue (dev, VideoMode, 0, slVideoModes);
   memcpy (pCam, pdefault_camdef+nVideoMode, sizeof (CAMTYPE_T));

   strncpy (pCam->Name, Name, CT_NAME_LENGTH);
   pCam->VideoMode      = nVideoMode;
   pCam->Interlaced     = GetStrValue (dev, Interlaced, 1, slInterlModes);
   pCam->VerticalFreq   = VFreq;
   pCam->HorizontalFreq = HFreq;
   pCam->ClockFreq      = ClockFreq;
   pCam->ClockMode      = GetStrValue (dev, ClockMode, 0, slClockModes);
}


/* ------------------------------ mvDefCamAcquireSetup ------------------ */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamAcquireSetup (DEV_T FAR* dev,
                                   char FAR* Name,
                                   char FAR* AcquireMode,
                                   char FAR* Invert,
                                   char FAR* StartMode)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
     pCam->AcquireMode = GetStrValue (dev, AcquireMode, 0, slAcquireModes);
     pCam->InvertSync  = GetStrValue (dev, Invert, 0, slInvertModes);
     pCam->StartMode   = GetStrValue (dev, StartMode, 3, slStartModes);
   }
}


/* ------------------------------ mvDefHorizontalUnit ------------------- */

WORD  _MVDFKT INTERPRETER_FKT mvDefHorizontalUnit (DEV_T FAR* dev,
                                  char FAR* Name,
                                  char FAR* Unit)
{
   WORD last = 0, wUnit;
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
     last = pCam->HTParameterMode;
     wUnit = GetStrValue (dev, Unit, 0, slParModes);

     switch (wUnit)
     {
       case 0:  /* Pixel */
         pCam->HTAcquireStart     = _GetHPix (pCam, pCam->HTAcquireStart);
         pCam->HTAcquireWidth     = _GetHPix (pCam, pCam->HTAcquireWidth);
         pCam->HTClampStart       = _GetHPix (pCam, pCam->HTClampStart);
         pCam->HTClampWidth       = _GetHPix (pCam, pCam->HTClampWidth);
         pCam->HTZeroStart        = _GetHPix (pCam, pCam->HTZeroStart);
         pCam->HTZeroWidth        = _GetHPix (pCam, pCam->HTZeroWidth);
         pCam->HTFieldStart       = _GetHPix (pCam, pCam->HTFieldStart);
         pCam->HTFieldWidth       = _GetHPix (pCam, pCam->HTFieldWidth);
         break;

      case 1:  /* Time */
         pCam->HTAcquireStart     = _GetHTime (pCam, pCam->HTAcquireStart);
         pCam->HTAcquireWidth     = _GetHTime (pCam, pCam->HTAcquireWidth);
         pCam->HTClampStart       = _GetHTime (pCam, pCam->HTClampStart);
         pCam->HTClampWidth       = _GetHTime (pCam, pCam->HTClampWidth);
         pCam->HTZeroStart        = _GetHTime (pCam, pCam->HTZeroStart);
         pCam->HTZeroWidth        = _GetHTime (pCam, pCam->HTZeroWidth);
         pCam->HTFieldStart       = _GetHTime (pCam, pCam->HTFieldStart);
         pCam->HTFieldWidth       = _GetHTime (pCam, pCam->HTFieldWidth);
         break;
     }

     pCam->HTParameterMode = wUnit;
   }

   return last;
}


/* ------------------------------ mvDefVerticalUnit --------------------- */

WORD  _MVDFKT INTERPRETER_FKT mvDefVerticalUnit (DEV_T FAR* dev,
                                char FAR* Name,
                                char FAR* Unit)
{
   WORD last = 0, wUnit;
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
     last  = pCam->VTParameterMode;
     wUnit = GetStrValue (dev, Unit, 2, slParModes);

     switch (wUnit)
     {
       case 2:  /* Lines */
         pCam->VTAcquireStart = _GetVLine (pCam, pCam->VTAcquireStart);
         pCam->VTAcquireWidth = _GetVLine (pCam, pCam->VTAcquireWidth);
         break;

      case 1:  /* Time */
         pCam->VTAcquireStart = _GetVTime (pCam, pCam->VTAcquireStart);
         pCam->VTAcquireWidth = _GetVTime (pCam, pCam->VTAcquireWidth);
         break;
     }

     pCam->VTParameterMode = wUnit;
   }

   return last;
}


/* ------------------------------ mvDefCamAnalogParam ------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamAnalogParam (DEV_T FAR* dev,
                                  char FAR* Name,
                                  char FAR* ChromFilter,
                                  WORD Gain,
                                  INT  OffsetVol,
                                  WORD LowRefVol,
                                  WORD HighRefVol)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->ChrominanceFilter = GetStrValue (dev, ChromFilter, 0, slFilterModes);
      pCam->Gain[0]           = Gain;
      pCam->Offset[0]         = OffsetVol;
      pCam->LowReference[0]   = LowRefVol;
      pCam->HighReference[0]  = HighRefVol;
   }
}


/* ------------------------------ mvDefChannelGain ---------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefChannelGain (DEV_T FAR* dev,
                               char FAR* Name,
                               WORD Channel,
                               WORD Gain)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      if (Channel < 32)
        pCam->Gain[Channel] = Gain;
   }
}


/* --------------------------------------------------- mvDefChannelOffset --- */
VOID  _MVDFKT INTERPRETER_FKT mvDefChannelOffset       ( DEV_T FAR* dev,
                                        char FAR* Name,
                                        WORD Channel,
                                        INT OffsetVol )
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr(dev,Name);

   if( pCam )
   {
      if( Channel < 32 )
          pCam->Offset[Channel]     = OffsetVol;
   }
}


/* ------------------------------ mvDefChannelReference ----------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefChannelReference (DEV_T FAR* dev,
                                    char FAR* Name,
                                    WORD Channel,
                                    WORD ReferenceNr,
                                    WORD RefVol)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      if (Channel < 32)
      {
         if (ReferenceNr == 0)
         {
            pCam->LowReference[Channel] = RefVol;
         }

         else if (ReferenceNr == 1)
         {
            pCam->HighReference[Channel] = RefVol;
         }
      }
   }
}


/* ------------------------------ mvDefCamHorizontalAcquire ------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamHorizontalAcquire (DEV_T FAR* dev,
                                        char FAR* Name,
                                        DWORD Start,
                                        DWORD Width,
                                        WORD Increment)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->HTAcquireStart     = Start;
      pCam->HTAcquireWidth     = Width;
      pCam->HTAcquireIncrement = Increment;
   }
}


/* ------------------------------ mvDefCamVerticalAcquire --------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamVerticalAcquire (DEV_T FAR* dev,
                                      char FAR* Name,
                                      DWORD Start,
                                      DWORD Width,
                                      WORD Increment)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->VTAcquireStart     = Start;
      pCam->VTAcquireWidth     = Width;
      pCam->VTAcquireIncrement = Increment;
   }
}


VOID _MVDFKT INTERPRETER_FKT mvDefCamGenerateVSync ( DEV_T FAR* dev,
                                  char FAR* Name,
                                  DWORD Start,
                                  DWORD Width )
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->GenerateVSync      =  1;
      pCam->GenerateVSyncStart = Start;
      pCam->GenerateVSyncWidth = Width;
   }

}


/* ------------------------------ mvDefCamClamp ------------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamClamp (DEV_T FAR* dev,
                            char FAR* Name,
                            DWORD Start,
                            DWORD Width)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->HTClampStart = Start;
      pCam->HTClampWidth = Width;
   }
}


/* ------------------------------ mvDefCamZero -------------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamZero (DEV_T FAR* dev,
                           char FAR* Name,
                           DWORD Start,
                           DWORD Width)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->HTZeroStart = Start;
      pCam->HTZeroWidth = Width;
   }
}


/* ------------------------------ mvDefCamFieldGate --------------------- */

VOID  _MVDFKT INTERPRETER_FKT mvDefCamFieldGate (DEV_T FAR* dev,
                                char FAR* Name,
                                DWORD Start,
                                DWORD Width)
{
   LPCAMTYPE_T pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
      pCam->HTFieldStart = Start;
      pCam->HTFieldWidth = Width;
   }
}


/* ------------------------------ mvGetCameraTypePtr -------------------- */

LPCAMTYPE_T  _MVDFKT mvGetCameraTypePtr (DEV_T FAR* dev, char FAR* Name)
{
   LPCAMTYPE_T pCam = pFirstCamType;
   LPCAMTYPE_T pResult = NULL;
   unsigned int i;

   UNUSED (dev);

   for (i = 0; i < NumCamTypes; i++)
   {
      if (stricmp (pCam->Name, Name) == 0)
      {
         pResult = pCam;
         break;
      }

      pCam++;
   }

   return pResult;
}


/* ------------------------------ mvGetActCameraTypePtr ----------------- */

LPCAMTYPE_T  _MVDFKT mvGetActCameraTypePtr (DEV_T FAR* dev)
{
   UNUSED (dev);
   return pActCamType;
}


/* ------------------------------ mvGetFirstCameraTypePtr --------------- */
LPCAMTYPE_T  _MVDFKT mvGetFirstCameraTypePtr (DEV_T FAR* dev)
{
   UNUSED(dev);
   return pFirstCamType;
}


/* ------------------------------ mvGetNumCameraTypesDefined ------------ */
WORD  _MVDFKT mvGetNumCameraTypesDefined (DEV_T FAR* dev)
{
   UNUSED(dev);
   return NumCamTypes;
}


/* ------------------------------ LOCAL: GetOffsetValue ----------------- */
static WORD GetOffsetValue(LPCAMTYPE_T pCam, WORD DigType, WORD channel)
{
   float Offset_mV = (float) pCam->Offset[channel];
   WORD Offset = 2048;

   switch ( DigType )
   {
   case DIG_MGVS:       /* PCprocess with MGVS-Module */
   case DIG_MRGB:       /* PCprocess with MRGB-Module */
      Offset = (WORD) (Offset_mV*2048.0/1200.0 + 2048.0);
      break;

   case DIG_MGVSi:      /* PCimage */
      Offset = (WORD) (Offset_mV*2048.0/864.0 + 2048.0);
      break;

   case DIG_PCISG:      /* PCimage-SG/SGVS & SRGB */
   case DIG_PCISRGB:  
      Offset = (WORD) (Offset_mV*2048.0/1200.0 + 2048.0);
      break;

   case DIG_PCISC:      /* PCimage-SC */
      Offset = (WORD) (Offset_mV*2048.0/1200.0 + 2048.0);
      break;

   }

   return Offset;

}


/* ------------------------------ LOCAL: GetReferenceValue -------------- */
static WORD GetReferenceValue(LPCAMTYPE_T pCam, WORD DigType, WORD channel, WORD type )
{
   float Reference_mV;
   WORD Reference;

   if( type == 0 )
   {
     Reference_mV = (float) pCam->LowReference[channel];
     Reference = 0;
   }
   else
   {
     Reference_mV = (float) pCam->HighReference[channel];
     Reference = 63;
   }


   switch ( DigType )
   {
   case DIG_MGVS:   /* PCprocess with MGVS-Module */
   case DIG_MRGB:   /* PCprocess with MRGB-Module */
      Reference = (WORD) ( Reference_mV / 31.31 );
      break;

   case DIG_MGVSi:  /* PCimage */
      Reference = (WORD) ( Reference_mV / 38.8 );
      break;

   case DIG_PCISG:  /* PCimage-SG/SGVS & SRGB */
   case DIG_PCISRGB:
      Reference = (WORD) ( Reference_mV / 38.8 );
      break;

   case DIG_PCISC:  /* PCimage-SC */
      Reference = (WORD) ( Reference_mV / 38.8 );
      break;

   }

   return Reference;
}


static IsSx ( DEV_T FAR* dev )
{
    switch ( dev->Glob->DevType )   
    {
        case DEV_PCISC :    
        case DEV_PCISG : 
        case DEV_PCISRGB : 
        case DEV_PCISDIG :  return TRUE;
        default :           return FALSE ;
    }
}

static WORD GetDigType( DEV_T FAR* dev )
{
   ACQUIRE_DEF_T acq;


   switch ( dev->Glob->DevType )
   {
        default :
        case DEV_PCISC :    return DIG_PCISC;
        case DEV_PCISG :    return DIG_PCISG;
        case DEV_PCISRGB :  return DIG_PCISRGB;
        case DEV_PCISDIG :  return DIG_PCISDIG;

        case DEV_PCIB :     return DIG_MGVSi;

        case DEV_PCPM :     mvGetAcquireDef (dev, &acq);
                            switch ((BYTE) (acq.start_addr[0] >> 24))
                            {
                                default :
                                case 0xBE:  return DIG_MGVS;
                                case 0xBF:  return DIG_MRGB;
                            }
    }
}



VOID  _MVDFKT mvUpdateActCamera ( DEV_T FAR* dev, DWORD lUpdateFlags )
{
   mvUpdateCamera ( dev, pActCamType, lUpdateFlags ) ;
}


VOID  _MVDFKT mvUpdateCamera ( DEV_T FAR* dev, LPCAMTYPE_T pCam, DWORD lUpdateFlags )
{
   ACQUIRE_DEF_T acq;
   DWORD         AcquireStartAddr, ltmp;
   WORD          xsize, ysize,wtmp, flags;
   WORD          DigType;
   INT           xdummy;
   int           i;


   DigType = GetDigType(dev);

   if( lUpdateFlags & CAM_UPDATE_VIDEO )
   {

     /* ------------ Select Standard Videomode */
     if (pCam->AcquireMode ==  0)
     {
      mvSelVideoMode (dev, pCam->VideoMode);
     }
     else
     {
        flags = 0;

        if ( pCam->InvertSync & SYNCINV_HV )
           flags |= VM_INVVSCAN;

        if ( pCam->InvertSync & SYNCINV_H )
           flags |= VM_INVHSYNC;

        if ( pCam->InvertSync & SYNCINV_V )
           flags |= VM_INVVSYNC;

       mvSelVideoMode (dev, (WORD) (pCam->VideoMode | flags));
     }

     /* ------------ AcquireDef */
     mvGetAcquireDef (dev, &acq);

     switch (DigType)
     {
       case DIG_MGVS:
                   xdummy = 4;
                   break;

       case DIG_MRGB:
                   xdummy = 0;
                   break;

       default:
                   xdummy = 0;
                   break;
     }

     AcquireStartAddr = min(acq.start_addr[1],acq.start_addr[0]);

     xsize = (WORD) (4 * (_HP (HTAcquireWidth) >> 2) / pCam->HTAcquireIncrement);
     ysize = (WORD) (_VL (VTAcquireWidth) / pCam->VTAcquireIncrement);
/*
     xsize = min (XSIZE (dev), xsize);
     ysize = min (YSIZE (dev), ysize);
*/
     if (pCam->Interlaced == 1)  /* Interlaced */
     {
       acq.fields_per_frame = 2;
       if( (acq.start_addr[0] < acq.start_addr[1]) || (acq.ypos[0] < acq.ypos[1]) )
       {
          ltmp = acq.start_addr[0];
          acq.start_addr[0] = acq.start_addr[1];
          acq.start_addr[1] = ltmp;

          wtmp = acq.ypos[0];
          acq.ypos[0] = acq.ypos[1];
          acq.ypos[1] = wtmp;
       }
     }
     else if ( pCam->Interlaced == 2 ) /* Field inverted interlaced */
     {
       acq.fields_per_frame = 2;
       if( (acq.start_addr[0] > acq.start_addr[1]) || (acq.ypos[0] > acq.ypos[1]) )
       {
          /* swap start address of the fields */
          ltmp = acq.start_addr[1];
          acq.start_addr[1] = acq.start_addr[0];
          acq.start_addr[0] = ltmp;

          wtmp = acq.ypos[0];
          acq.ypos[0] = acq.ypos[1];
          acq.ypos[1] = wtmp;
       }
     }
     else   /* Noninterlaced */
     {
       acq.fields_per_frame = 1;

       if ( DigType == DIG_PCISG || DigType == DIG_PCISRGB )
       {
           acq.trig_mode |= TM_DISABLE_FIELD_DETECT;
       }
       else
       {
          acq.line_inc[0] /= 2;
          acq.line_inc[1] /= 2;
       }

       acq.start_addr[0] = AcquireStartAddr;
       acq.start_addr[1] = AcquireStartAddr;
       acq.ypos[0] = 0;
       acq.ypos[1] = 0;
     }


     acq.trig_mode &= ~3;
     acq.trig_mode |= pCam->StartMode;
     acq.vcnt_start = (WORD) (VCNT_START (xsize, _VL (VTAcquireStart)));
     acq.vcnt_incr  = pCam->VTAcquireIncrement;

     for (i = 0; i < 2; i++)
     {
       acq.xsize[i] = xsize;
       acq.ysize[i] = ysize;
     }

     mvSetAcquireDef (dev, &acq);

     /* --------------- Timing RAM */
     {
         WORD xs, ys, xe, ye ;

         xs = (WORD) _HP (HTAcquireStart) ;
         xe = (WORD) (xs + (xsize + xdummy -1) * pCam->HTAcquireIncrement) ;
         ys = (WORD) _VL (VTAcquireStart) ;
         ye = (WORD) (_VL (VTAcquireStart) + ysize -1) ;



         mvSetVideoTiming (dev, HRAM, 0, 4095, 1, HVT_ACQUIRE, BIT_RESET);
         mvSetVideoTiming (dev, HRAM, xs, xe, pCam->HTAcquireIncrement, HVT_ACQUIRE, BIT_SET);

         mvSetVideoTiming (dev, VRAM, 0, 4095, 1, HVT_ACQUIRE, BIT_RESET);
         mvSetVideoTiming (dev, VRAM, ys, ye, 1, HVT_ACQUIRE, BIT_SET);

         mvSetVideoTiming (dev, VRAM, 0, 4095, 1, HVT_SYNC, BIT_RESET);

         if ( pCam->GenerateVSync  )
         {
            mvSetVideoTiming (dev, VRAM, (WORD) (_VL (GenerateVSyncStart) ), (WORD) (_VL (GenerateVSyncStart) + _VL (GenerateVSyncWidth) - 1 ), 1, HVT_SYNC, BIT_SET);
         }
     }

   }


   if( lUpdateFlags & CAM_UPDATE_ZERO )
   {
     mvSetVideoTiming (dev, HRAM, 0, 4095, 1, HVT_ZERO, BIT_RESET);
     if ((pCam->HTZeroWidth > 0) && (DigType != DIG_MGVSi))
       mvSetVideoTiming (dev, HRAM, (WORD) _HP (HTZeroStart), (WORD) (_HP (HTZeroStart) + _HP (HTZeroWidth)), 1, HVT_ZERO, BIT_SET);
   }

   if( lUpdateFlags & CAM_UPDATE_CLAMP )
   {
     mvSetVideoTiming (dev, HRAM, 0, 4095, 1, HVT_CLAMP, BIT_RESET);
     if (pCam->HTClampWidth > 0)
       mvSetVideoTiming (dev, HRAM, (WORD) _HP (HTClampStart), (WORD) (_HP (HTClampStart) + _HP (HTClampWidth)), 1, HVT_CLAMP, BIT_SET);
   }

   if( lUpdateFlags & CAM_UPDATE_FIELD )
   {
     mvSetVideoTiming (dev, HRAM, 0, 4095, 1, HVT_FIELD, BIT_RESET);
     if (pCam->HTFieldWidth > 0)
       mvSetVideoTiming (dev, HRAM, (WORD) _HP (HTFieldStart), (WORD) (_HP (HTFieldStart) + _HP (HTFieldWidth)), 1, HVT_FIELD, BIT_SET);
   }


   if( lUpdateFlags & CAM_UPDATE_VIDEO )
   {
     mvLoadTimingRAM (dev);
   }

   if ( ( lUpdateFlags & CAM_UPDATE_CHROMFILTER ) != 0
     && ! ( dev->Glob->DevType == DEV_PCISRGB
                && ( pCam->ChrominanceFilter == 1 || pCam->ChrominanceFilter == 2 ) ) )
   {
     mvSelChrominanceFilter (dev, pCam->ChrominanceFilter);

     if (GetErr (dev) == ERR_NOT_IMPLEMENTED)     /* AB: Dig-Modul hat keine Fkt. SelChrominanceFilter */
       ClearErr (dev);
   }


   if( lUpdateFlags & CAM_UPDATE_GAIN )
   {
     mvSetGain (dev, 0, pCam->Gain[0]);

     if (GetErr (dev) == ERR_NOT_IMPLEMENTED)     /* AB: Dig-Modul hat keine Fkt. SetGain */
       ClearErr (dev);

   }

   if (dev->Glob->DevType != DEV_PCISC)           /* AB: PCimageSC hat keinen externen Pixeltakt */ 
   {
     if (lUpdateFlags & (CAM_UPDATE_CLOCKMODE | CAM_UPDATE_CLOCKFREQ | CAM_UPDATE_INVERTSYNC))
     {
       if (pCam->ClockMode == 0)
       {
         mvSelInputClk (dev, 0);
         mvSetPixelFreq (dev, _PClk (pCam));
       }
       else
       {

		 if (pCam->InvertSync & 1)
            if ( dev->Glob->DevType == DEV_PCISG || dev->Glob->DevType == DEV_PCISRGB )
		       mvSelInputClk (dev,(WORD)-1);
			else
		       mvSelInputClk (dev, 2);
		 else
		   mvSelInputClk (dev, 1);

       }
     }
   }

   if( lUpdateFlags & CAM_UPDATE_OFFSET )
   {
     WORD offset = GetOffsetValue(pCam,DigType,0) ;
     mvSetOffset(dev,0, offset );
	 if (GetErr (dev) == ERR_NOT_IMPLEMENTED)     /* HB: Dig-Modul hat keine Fkt. SetGain */
       ClearErr (dev);

   }

#if 0
   if ( ! IsSx(dev) )        /* WA: Sx returns error */
   {

     if( lUpdateFlags & CAM_UPDATE_LOWREF )
     {

       if( DigType != DIG_MGVSi )
       {
         WORD ref = GetReferenceValue(pCam,DigType,0,0) ;
         mvSetReference( dev,0,0, ref );
       }
     }

     if( lUpdateFlags & CAM_UPDATE_HIREF )
     {
       if( DigType != DIG_MGVSi )
       {
         WORD ref = GetReferenceValue(pCam,DigType,0,1) ;
         mvSetReference( dev,0,1, ref );
       }
     }
   }

#endif




}



WORD  _MVDFKT INTERPRETER_FKT mvSelCamera (DEV_T FAR* dev, char FAR* Name)
{
   LPCAMTYPE_T   pCam;

   pCam = mvGetCameraTypePtr (dev, Name);

   if (pCam)
   {
     pActCamType = pCam;
     mvUpdateActCamera( dev, CAM_UPDATE_ALL );
   }

   return 0;
}


VOID _MVDFKT mvOutputCamType ( LPCAMTYPE_T pCam, PRINTFKT prf )
{
   if (pCam)
   {
     (*prf) ("----------- Cameratype: %s ------------\n", pCam->Name);
     (*prf) ("VideoMode            %s\n", slVideoModes[pCam->VideoMode]);
     (*prf) ("Interlaced           %s\n", slInterlModes[pCam->Interlaced]);
     (*prf) ("VerticalFreq         %d Hz\n", _VClk (pCam));
     (*prf) ("HorizontalFreq       %ld Hz\n", (long)(_HClk (pCam) * 1000.0 ) );
     (*prf) ("ClockFreq            %ld kHz\n", (long)(_PClk (pCam) * 1000.0 ) );
     (*prf) ("ClockMode            %d\n", pCam->ClockMode);
     (*prf) ("AcquireMode          %s\n", slAcquireModes[pCam->AcquireMode]);
     (*prf) ("InvertSync           %d\n", pCam->InvertSync);
     (*prf) ("StartMode            %s\n", slStartModes[pCam->StartMode]);
     (*prf) ("ChrominanceFilter    %s\n", slFilterModes[pCam->ChrominanceFilter]);
     (*prf) ("Gain:                %d\n", pCam->Gain);
     (*prf) ("Offset:              %d mV\n", pCam->Offset);
     (*prf) ("LowReference:        %d mv\n", pCam->LowReference);
     (*prf) ("HighReference:       %d mv\n\n", pCam->HighReference);

     (*prf) ("Horizontal Acquire:  %4ld (%6ld ns)   %4ld (%6ld ns)   %d\n",
                                   _GetHPix (pCam, pCam->HTAcquireStart),_GetHTime(pCam,pCam->HTAcquireStart),
                                   _GetHPix (pCam, pCam->HTAcquireWidth), _GetHTime (pCam, pCam->HTAcquireWidth), 
                                   pCam->HTAcquireIncrement);
     (*prf) ("Clamp:               %4ld (%6ld ns)   %4ld (%6ld ns)\n",
                                    _GetHPix (pCam, pCam->HTClampStart), _GetHTime (pCam, pCam->HTClampStart),
                                    _GetHPix (pCam, pCam->HTClampWidth), _GetHTime (pCam, pCam->HTClampWidth));
     (*prf) ("Zero:                %4ld (%6ld ns)   %4ld (%6ld ns)\n\n", _GetHPix (pCam, pCam->HTZeroStart), _GetHTime (pCam, pCam->HTZeroStart), _GetHPix (pCam, pCam->HTZeroWidth), _GetHTime (pCam, pCam->HTZeroWidth));

     (*prf) ("Vertical Acquire:    %4ld (%6ld us)   %4ld (%6ld us)   %d\n",
                                            _GetVLine (pCam, pCam->VTAcquireStart), _GetVTime (pCam, pCam->VTAcquireStart),
                                            _GetVLine (pCam, pCam->VTAcquireWidth), _GetVTime (pCam, pCam->VTAcquireWidth),
                                            pCam->VTAcquireIncrement);
   }
}

/* ------------------------------ mvPrintCamType ------------------------ */
#if !defined(__mvWIN__) 
VOID _MVDFKT mvPrintCamType (DEV_T FAR* dev, char FAR* Name)
{
   mvOutputCamType ( mvGetCameraTypePtr (dev, Name), printf ) ;
}
#endif

/* Test: */
/*
void main()
{
   LPCAMTYPE_T pCam;

   mvInitCamTypeDefinitions();

   mvDefCamType (NULL, "XC-75","VM_CCIR","INTERLACED", 50, 15625, 15000, 0);
   mvDefCamVerticalAcquire ( NULL, "XC-75", 25L, 574L, 1, "LINES");

   mvDefCamType (NULL, "XC-77","VM_RGB","NONINTERLACED", 60, 15000, 16000, 0);
   mvDefCamType (NULL, "123","VM_RGB","NONINTERLACED", 60, 15000, 16000, 0);


   mvPrintCamType(NULL, "XC-75");

   mvFreeCamTypeDefinitions();

}

*/



