#include <vcl.h>

#define CONVVERS "1.2 (09/10/2004)" /* Version of this program */

/* DIG header format: */
#define TRI                   "DIG" /* Header beginning sign */
#define DIG_HEADER_SIZE         88  /* Size of header */
#define HEAD_LENGTH              3  /* Length of TRI */
#define COPYRIGHT "DIG picture file - (C) 2002 J. S., S. D."   /* DIG (C) */
#define COPY_LENGTH             40  /* Length of COPYRIGHT */
#define DIG_VERSION              2  /* Version of DIG file */
#define COMPRESS_ENGINE_VERSION  0  /* Compression of DIG */
#define COMPRESS_CHAR           32  /* ' ' is sign of no compression */
typedef struct {
  char tri[HEAD_LENGTH+1];
  signed int version;
  char compressed;
  signed int compr_eng_ver;
  char copyright[COPY_LENGTH+1];
  signed int voss_width;
  signed int voss_height;
  signed int year;
  signed int month;
  signed int day;
  signed int hour;
  signed int minute;
  signed int second;
  signed int ssecond;
} t_dig_header;
#define HEADER_TEST_VARIABLES  12  /* Number of DIG header variables to test */

/* FITS format: */
#define F_BLOCK_SIZE         2880  /* Length of the header */
#define BLOCK_HALF  (F_BLOCK_SIZE / 2)  /* Length of buffer in pixels */

#define F_SUB_BLOCK_SIZE       30  /* Length of subblock in header item; new in 1.1 */
#define F_HEADER_BLOCK         80  /* Length of one header item */

#define SPACE                  32  /* Empty space character */

/*
 * Digfit main program file.
 *
 * Version 1.2, 09/10/2004
 *
 * This file is designed for use in Wondows(R) systems.
 * Tested with compiler: GCC v3.2 (MinGW special 20020817-1).
 *
 * Copyright (C) 2004 Jiri Stepan (jiri.stepan@volny.cz).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/stat.h>

/* Print usage on screen */
void printHelp()
{
  ShowMessage("Pouziti:  digfit <[cesta]input.dig> <[cesta]output.fit> [-h]\n\n");
  ShowMessage("  <input.dig>: vstupni soubor formatu DIG verze 2\n");
  ShowMessage("  <output.fit>: vystupni soubor formatu FITS (16-bit unsigned integer)\n");
  ShowMessage("  [cesta]: nepovinna cesta k adresarum obsahujicim vstupni/vystupni soubor\n");
  ShowMessage("  [-h]: tato napoveda\n\n");
  ShowMessage("Kontakt: Jiri Stepan (jiri.stepan@volny.cz), 2004.\n\n");
}

/* Test DIG header for consistecy and version */
int testHeader(t_dig_header *Header)
{
  int v[HEADER_TEST_VARIABLES] =
    {
     /* 0 */  strcmp(Header->tri,TRI),
     /* 1 */  (Header->version != DIG_VERSION),
     /* 2 */  (Header->compressed != COMPRESS_CHAR),
     /* 3 */  (Header->compr_eng_ver != COMPRESS_ENGINE_VERSION),
     /* 4 */  strcmp((char*)&Header->copyright,COPYRIGHT),
     /* 5 */  ((Header->voss_width <= 0) || (Header->voss_height <= 0)),
     /* 6 */  ((Header->month < 1) || (Header->month > 12)),
     /* 7 */  ((Header->day < 1) || (Header->day > 31)),
     /* 8 */  ((Header->hour < 0) || (Header->hour > 23)),
     /* 9 */  ((Header->minute < 0) || (Header->minute > 59)),
     /* 10 */ ((Header->second < 0) || (Header->second > 59)),
     /* 11 */ ((Header->ssecond < 0) || (Header->ssecond > 999))
    };
  int j;            /* Index variable */
  int result = 0;   /* We return ! of this variable */
  for (j=0; j<HEADER_TEST_VARIABLES; j++)
  {
    result = result || v[j];  /* Test of zero value of all v[] variables */
  }
#ifdef DEBUG
# define TISKi(x) ShowMessage("%s = %d\n",#x,x);
# define TISKs(x) ShowMessage("%s = %s\n",#x,x);
  TISKs(Header->tri);
  TISKi(Header->version);
  TISKi(Header->compressed);
  TISKi(Header->compr_eng_ver);
  TISKs(Header->copyright);
  TISKi(Header->voss_width);
  TISKi(Header->voss_height);
  TISKi(Header->month);
  TISKi(Header->day);
  TISKi(Header->hour);
  TISKi(Header->minute);
  TISKi(Header->second);
  TISKi(Header->ssecond);
  int i;
  for (i=0; i<12; i++)
  {
    if (v[i])
    {
      ShowMessage("CHYBA V POLOZCE C. %d!\n",i);
    }
  }
#  undef TISKi
#  undef TISKs
#endif // DEBUG
  return !result;
}

/* Print picture informations on screen */
void printStat(t_dig_header *Header, char* inpf, char* outf) {
  int sizeDIG = DIG_HEADER_SIZE + Header->voss_width * Header->voss_height * 2,
      sizex   = sizeDIG - DIG_HEADER_SIZE + F_BLOCK_SIZE,
      sizeFIT = F_BLOCK_SIZE - sizex % F_BLOCK_SIZE + sizex;
  /*ShowMessage("`%s\' (%d B) -> `%s\' (%d B)\n",inpf,sizeDIG,outf,sizeFIT);
  ShowMessage("Verze DIG: %d\n",Header->version);
  ShowMessage("Komprese DIG: NE\n");
  ShowMessage("Rozmery (S x V): %d x %d\n",Header->voss_width,Header->voss_height);
  ShowMessage("Cas porizeni: %d.%d.%d; %d:%d:%d.%d\n",Header->day,Header->month,
         Header->year,Header->hour,Header->minute,Header->second,Header->ssecond);*/
}

/* Swap bytes of 2-byte long word */
inline unsigned int swap_bytes(unsigned int x)
{
  return (x << 8) + (x >> 8);
}

/* Aligns 'left' text on left, 'right' on right in 'buffer';
   New in version 1.1 */
void alignText(char *left, char *right, char *buffer, int bufferLength)
{
  int leftLength  = strlen(left),
      rightLength = strlen(right);
  memcpy(buffer,left,leftLength);
  memcpy(buffer+bufferLength-rightLength,right,rightLength);
}

/* Convert DIG to FITS */
#define ERR_OUTP {ShowMessage("Nelze zapisovat do souboru!"); \
                  fclose(f_in); \
                  fclose(f_ou); \
                  remove(out); \
                  return 0;}
int convertFile(char *inp, char *out)
{
  /* Open input and output files */
  FILE *f_in = fopen(inp,"rb"),
       *f_ou = fopen(out,"wb");
  /* Input file not opend? */
  if (!f_in)
  {
    ShowMessage("Soubor " + (AnsiString)inp + " se nepodarilo otevrit!");
    /* Remove existing output file and exit */
    if (f_ou)
    {
      fclose(f_ou);
      remove(out);
    }
    return 0;
  }
  /* Output file not opend? */
  if (!f_ou)
  {
    ShowMessage("Soubor " + (AnsiString)inp + " se nepodarilo otevrit!");
    fclose(f_in);  /* Close opend input file and exit */
    return 0;
  }
  /* Read DIG header */
  t_dig_header DIGheader;
  DIGheader.copyright[COPY_LENGTH] = DIGheader.tri[HEAD_LENGTH] = 0;
  if (
    fread(&DIGheader.tri,HEAD_LENGTH,1,f_in) &
    fread(&DIGheader.version,4,1,f_in) &
    fread(&DIGheader.compressed,1,1,f_in) &
    fread(&DIGheader.compr_eng_ver,4,1,f_in) &
    fread(&DIGheader.copyright,COPY_LENGTH,1,f_in) &
    fread(&DIGheader.voss_width,4,1,f_in) &
    fread(&DIGheader.voss_height,4,1,f_in) &
    fread(&DIGheader.year,4,1,f_in) &
    fread(&DIGheader.month,4,1,f_in) &
    fread(&DIGheader.day,4,1,f_in) &
    fread(&DIGheader.hour,4,1,f_in) &
    fread(&DIGheader.minute,4,1,f_in) &
    fread(&DIGheader.second,4,1,f_in) &
    fread(&DIGheader.ssecond,4,1,f_in) != 1 )
  {
      ShowMessage("Soubor " + (AnsiString)inp + " se nepodarilo otevrit!");
      /* Colse files, remove existing output file and exit */
      fclose(f_in);
      fclose(f_ou);
      remove(out);
      return 0;
  }
  /* Test header and file size consistency */
  struct stat inp_stat;
  stat(inp,&inp_stat);
  unsigned int inp_size = (unsigned int)inp_stat.st_size; /* Size of DIG file */
  if (!testHeader(&DIGheader) ||
      (inp_size != DIG_HEADER_SIZE +
                   2 * DIGheader.voss_width * DIGheader.voss_height))
  {
    /* Error occured -> exit */
    ShowMessage("Format souboru " + (AnsiString)inp + " neodpovida definici DIG!");
    fclose(f_in);
    fclose(f_ou);
    remove(out);
    return 0;
  }
  /* Write header of FITS */
  int cnt = 0;  /* Counter */
  char buff[F_HEADER_BLOCK];  /* Buffer for header 80-characters lines */
  /* Write FITS header */
  memset(&buff,SPACE,F_HEADER_BLOCK); /* Fill buffer by spaces */
  alignText("SIMPLE  =","T",&buff[0],F_SUB_BLOCK_SIZE); /* Simple FITS */
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* Bit depth */
  memset(&buff,SPACE,F_HEADER_BLOCK);
  alignText("BITPIX  =","16",&buff[0],F_SUB_BLOCK_SIZE); /* Gray depth = 2 bytes */
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* Number of axes */
  memset(&buff,SPACE,F_HEADER_BLOCK);
  alignText("NAXIS   =","2",&buff[0],F_SUB_BLOCK_SIZE); /* Two dimensional data */
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* Image width and height */
  char textNumber[F_HEADER_BLOCK];  /* Buffer for saving numbers in text form */
  memset(&buff,SPACE,F_HEADER_BLOCK);  /* Horizontal resolution... */
  sprintf(&textNumber[0],"%d",DIGheader.voss_width);
  alignText("NAXIS1  =",&textNumber[0],&buff[0],F_SUB_BLOCK_SIZE);
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  memset(&buff,SPACE,F_HEADER_BLOCK);  /* Vertical resolution... */
  sprintf(&textNumber[0],"%d",DIGheader.voss_height);
  alignText("NAXIS2  =",&textNumber[0],&buff[0],F_SUB_BLOCK_SIZE);
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* Date and time of observation */
  sprintf((char*)&buff,"DATE-OBS=\'%d%d/%d%d/%d%d\'",
          DIGheader.day/10,DIGheader.day%10,
          DIGheader.month/10,DIGheader.month%10,
          (DIGheader.year%100)/10,DIGheader.year%10);  /* Date of observation */
  memset((char*)&buff+strlen((char*)&buff),SPACE,F_HEADER_BLOCK-strlen((char*)&buff));
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  sprintf((char*)&buff,"TIME    =\'%d%d:%d%d:%d%d.%d%d%d\'",
          DIGheader.hour/10,DIGheader.hour%10,DIGheader.minute/10,DIGheader.minute%10,
          DIGheader.second/10,DIGheader.second%10,DIGheader.ssecond/100,
          (DIGheader.ssecond/10)%10,DIGheader.ssecond%10); /* Time of observation */
  memset((char*)&buff+strlen((char*)&buff),SPACE,F_HEADER_BLOCK-strlen((char*)&buff));
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* Some comment */
/*  sprintf((char*)&buff,"COMMENT Soubor vytvoren z predlohy `%s\' "
                       "ve formatu DIG programem `DIGFIT\'",inp); */
  sprintf((char*)&buff,"COMMENT Soubor vytvoren z predlohy ve formatu DIG programem CAM.",inp);
  memset((char*)&buff+strlen((char*)&buff),SPACE,F_HEADER_BLOCK-strlen((char*)&buff));
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  /* End of header */
  memset(&buff,SPACE,F_HEADER_BLOCK);
  memcpy(&buff,"END",3);
  if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1)
  {
    ERR_OUTP;
  }
  cnt+=9;  /* Number of written lines */
  /* Fill rest of the header with spaces */
  memset(&buff,SPACE,F_HEADER_BLOCK);
  for (; cnt<F_BLOCK_SIZE/F_HEADER_BLOCK; cnt++)
  {
    if (fwrite(&buff,F_HEADER_BLOCK,1,f_ou)!=1) ERR_OUTP;
  }
  /* Copy pixels data */
  int read = 0;  /* Number of read pixels */
  unsigned short int buf[BLOCK_HALF];  /* Buffer of pixels */
  /* Read and write data by 2880/2 pixels */
  while ((read=fread(&buf,2,BLOCK_HALF,f_in))!=0)
  {
    /* Scale down color value of each pixel in buffer */
    for (cnt=0; cnt<read; cnt++)
    {
      buf[cnt] >>= 4;
      buf[cnt] = swap_bytes(buf[cnt]);  /* Swap bytes in word */
    }
    /* Fill the rest of last buffer by zeros */
    if (read<BLOCK_HALF)
    {
        memset((unsigned short int*)&buf+read,0,2*(BLOCK_HALF-read));
    }
    /* Write buffer */
    if (fwrite(&buf,2,BLOCK_HALF,f_ou)!=BLOCK_HALF) ERR_OUTP;
  }
  /* Print statistics */
  printStat(&DIGheader,inp,out);
  /* Close files and exit */
  fclose(f_in);
  fclose(f_ou);
  return 1;
}
#undef ERR_OUTP

int convert_digfit(AnsiString file)
{
  int help = 0;  /* Implicitly don't print help */
  /* Convert file */
  AnsiString nname = file.SubString(17,10)+".fts";
  if (!convertFile(file.c_str(),nname.c_str()))
  {
    ShowMessage("Konverze souboru " + file + " se nezdaila!");
    return 1;
  }
  else
  {
    return 1;
  }
};

