/* DLLDRV.C dynamischen Laden der Treiber-DLL und bereitstellen der Funktionen */

#include <windows.h>
#define MV_DLL
#include "include\mv.h"
#include "dlldrv\camtype.h"

#define MAX_DEVICES 4

#define _mvOpenDevice       PCIDevice[m_SelDevNr].fkt.OpenDevice
#define _mvCloseDevice      PCIDevice[m_SelDevNr].fkt.CloseDevice
#define _mvLibInit          PCIDevice[m_SelDevNr].fkt.LibInit
#define _mvAddUserFunction  PCIDevice[m_SelDevNr].fkt.AddUserFunction
#define _mvGetIniFilename   PCIDevice[m_SelDevNr].fkt.GetIniFilename
#define _mvInterpretString  PCIDevice[m_SelDevNr].fkt.InterpretString
#define _mvInterpreter      PCIDevice[m_SelDevNr].fkt.Interpreter

#define _mvDev              PCIDevice[m_SelDevNr].PCIsxDev
#define _ErrTypeIndex(dev) (((dev)->ErrorCode>>14)&3)
#define _boLive             PCIDevice[m_SelDevNr].boLive
#define _hDLLModule         PCIDevice[m_SelDevNr].hDLLModule
#define _usedFlag           PCIDevice[m_SelDevNr].usedFlag


#define _BoardNotOpen       (m_SelDevNr<0)

typedef struct 
{
  DEV_T *  (FAR __stdcall * OpenDevice)( CHAR FAR*, CHAR FAR* FAR*, INT FAR*, DRVINI_T ) ;
  INT      (FAR __stdcall * CloseDevice)( DEV_T FAR* ) ;
  DRVINI_T (FAR __stdcall * LibInit)( DEV_T FAR*, CHAR FAR*, CHAR FAR* FAR* ) ;
  WORD     (FAR __stdcall * AddUserFunction )( const char FAR* FktName, USER_FUNC_T FktPtr );
  char  *  (FAR __stdcall * GetIniFilename )( DEV_T FAR* ) ;
  BOOL     (FAR __stdcall * InterpretString )( DEV_T FAR*, CHAR FAR* ) ;
  VOID     (FAR __stdcall * Interpreter ) (	DEV_T FAR*, char FAR*, char FAR*, PRINTFKT, GETSFKT ); 
} TDEVICE_FKT, *LPDEVICE_FKT;


typedef struct
{
   DEV_T        *PCIsxDev ;    // framegrabber device handle
   BOOL          usedFlag ;    // TRUE = this device handle is used
   TDEVICE_FKT   fkt;
   BOOL          boLive;
   HMODULE       hDLLModule;
} TDEVICES, *LPDEVICE;


static TDEVICES   PCIDevice[ MAX_DEVICES ] ; // struct array of device data

static short      m_SelDevNr = -1 ;          // currently selected device number
static BOOL       m_boModulInitialised = FALSE;

void InitModuldata()
{
  WORD w;
  if ( !m_boModulInitialised )
  {

    for ( w=0 ; w< MAX_DEVICES; w++ )
       PCIDevice[w].usedFlag = FALSE;

    m_boModulInitialised = TRUE;
  }
}

short FindDeviceNr( BOOL boUsed )
{
  short w;

  w = 0;
  while(  (PCIDevice[w].usedFlag == !boUsed) && (w<MAX_DEVICES) )
    w++;

  if ( w>=MAX_DEVICES )
    w = -1;

  return w;
}


short OpenDriverDLL( char* dllname )
{
  BOOL boOK = FALSE;
  short devnr;

  devnr = FindDeviceNr(FALSE);
  if ( devnr == -1 ) 
    return -1;

  m_SelDevNr = devnr;

  // Open frame grabber DLL
  _hDLLModule = LoadLibrary( dllname ) ;

  if ( _hDLLModule )
  {
      // get function pointer
      (FARPROC)_mvOpenDevice      = GetProcAddress( _hDLLModule, "mvOpenDevice" );
      (FARPROC)_mvCloseDevice     = GetProcAddress( _hDLLModule, "mvCloseDevice" );
      (FARPROC)_mvLibInit         = GetProcAddress( _hDLLModule, (LPCSTR) 4 );
      (FARPROC)_mvAddUserFunction = GetProcAddress( _hDLLModule, "mvAddUserFunction" );
      (FARPROC)_mvGetIniFilename  = GetProcAddress( _hDLLModule, "mvGetIniFilename" );
      (FARPROC)_mvInterpretString = GetProcAddress( _hDLLModule, "mvInterpretString" );
      (FARPROC)_mvInterpreter     = GetProcAddress( _hDLLModule, "mvInterpreter" );



      if (!_mvOpenDevice || !_mvCloseDevice || !_mvLibInit || !_mvAddUserFunction || !_mvGetIniFilename || !_mvInterpretString || !_mvInterpreter )
      {
/*
         char errortext[512] ;
         sprintf( errortext, "Driver-DLL function not found\n  %c mvOpenDevice\n  %c mvCloseDevice\n  %c mvLibInit\n  %c mvAddUserFunction\n  %c mvGetIniFilename\n  %c mvInterpretString",
           (_mvOpenDevice?'+':'-'),
           (_mvCloseDevice?'+':'-'),
           (_mvLibInit?'+':'-'),
           (_mvAddUserFunction?'+':'-'),
           (_mvGetIniFilename?'+':'-'),
           (_mvInterpretString?'+':'-') );

         mvOutputMessage( NULL, errortext, "Fatal Error", MB_OK | MB_ICONWARNING ) ;
*/
         return -1;
      }


  }


  return m_SelDevNr;

}



void CloseDriverDLL( )
{
  if ( _BoardNotOpen )
    return;


  if ( _hDLLModule )
  {
    FreeLibrary(_hDLLModule);

    _hDLLModule = NULL;
    _usedFlag = FALSE;

  }
}


//--------------- external functions ----------------


BOOL  ReadDllNameFromIni( char* IniFile, char *section, char * dllname )
{
  return GetPrivateProfileString( section, "Driver","",dllname,  _MAX_PATH, IniFile 	) > 0;	
}


short SelectBoard( DEV_T FAR* dev )
{
  short w;

  w = 0;
  while( (PCIDevice[w].PCIsxDev == dev) && (PCIDevice[w].usedFlag == TRUE) && (w<MAX_DEVICES) )
    w++;

  if ( w>=MAX_DEVICES )
    w = -1;
  else
    m_SelDevNr = w;

  return w;
}



DEV_T FAR* OpenDevice   /* initialize device and return device-key */
           (CHAR FAR* dllname_par,     /* name of DLL-file */
            CHAR FAR* inifile,     /* name of INI-file */
            INT FAR* result)       /* result */
{
	BOOL boDllExist = FALSE;

	char dllname[_MAX_PATH];
  
  if ( dllname_par == NULL ) 
  {
	  if (!ReadDllNameFromIni(inifile,"FrameGrabber",(char *)dllname))
	  {
		  *result = ERR_OPEN_FILE;
		  return NULL;
	  }
  }
  else
	  strcpy(dllname,dllname_par);

  if ( OpenDriverDLL( dllname ) >= 0 )
  {
    mvInitCamTypeDefinitions( 500 );
    return _mvOpenDevice(inifile,NULL,result, (DRVINI_T) _mvLibInit ); /* opens device */
  }
	
  return NULL;
}



short CloseDevice() /* closes device */
{
  short devnr;

  if ( _BoardNotOpen )
    return -1;

  mvFreeCamTypeDefinitions ();
  _mvCloseDevice( _mvDev );

  CloseDriverDLL();

  // select other used board
  devnr = FindDeviceNr(TRUE);

  return devnr;
}



_MVAPI(WORD) mvAddUserFunction
           (const char FAR* FktName,
            USER_FUNC_T FktPtr)
{
  if ( _BoardNotOpen )
    return 0;

  return _mvAddUserFunction(FktName,FktPtr);

}


_MVAPI(BOOL) mvInterpretString (DEV_T FAR* dev, CHAR FAR* str)
{
  if ( _BoardNotOpen )
    return FALSE;

  return _mvInterpretString(dev,str);
}

_MVAPI(char FAR*) mvGetIniFilename(DEV_T FAR* dev)
{
  if ( _BoardNotOpen )
    return NULL;

  return _mvGetIniFilename(dev);
}



_MVAPI(VOID)  mvInterpreter
           (DEV_T FAR* dev,
            char FAR* promtstr,
            char FAR* exitstr,
            PRINTFKT prnt,
            GETSFKT gts)
{

   if ( _BoardNotOpen )
    return;

   _mvInterpreter(dev,promtstr, exitstr, prnt, gts);
}
