/**************************************************************************
 *
 *  Copyright (C) 2002 Jiri Stepan <jiri.stepan@volny.cz>.
 *  All Rights Reserved.
 *
 **************************************************************************/
/**************************************************************************
 *
 *  main.cpp: Main unit for CAM
 *
 *  CAM Source code
 *
 **************************************************************************/


#include <vcl.h>
#pragma hdrstop

#include "main.h"
#include "sdigw.h"
#include "setupdir.h"
#include "dlgbrws.h"
#include "dlgsdig.h"
#include "about.h"

#include "include\mv.h"
#include "dlldrv\dlldrv.h"
#include "include\init.h"
#include "include\mverr.h"
#include "include\disp.h"
#include "include\mody.h"
#include "language.h"
#include "digfit.h"
#include "digfitconvert.h"

#include <sys\types.h>
#include <inifiles.hpp>
#include <iostream.h>
#include <conio.h>
#include <stdlib.h>
#include <inifiles.hpp>
#include <time.h>
#include <classes.hpp>
#include <vfw.h>

#pragma package(smart_init)
#pragma link "CSPIN"
#pragma resource "*.dfm"


__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
  TIniFile *ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
  delete ini;
}

int g_loging;
unsigned __int16 * buffer;
int bufferpitch;
Graphics::TBitmap *pBitmap;
DEV_T FAR* dev = NULL;
int HEIGHT;
int *height_table = NULL;
int g_pr = 0;
int g_konec = 0;
int g_exit_win;
TForm1 *Form1;

AnsiString TForm1::GetStandartMsg(int handle) {
  int id = -1;
  for (int i=0; L_handles[i]; i++)
    if (L_handles[i] == handle) {
      id = i;
      continue;
    }
  if (id == -1) return "";
  return L_sentences[id];
}

AnsiString TForm1::GetMsg(int handle) {
  if (LanguageFile == "") return GetStandartMsg(handle);
  TIniFile *ini = new TIniFile(LanguageFile);
  AnsiString label = ini->ReadString("Header","Label","");
  if (label != L_LABEL) {
    delete ini;
    LanguageFile = "";
    ShowMessage("Invalid language file "+LanguageFile+", using default language");
    label = GetStandartMsg(handle);
    return label;
  };
  if (!ini->ValueExists("Messages",IntToStr(handle))) {
    delete ini;
    LanguageFile = "";
    ShowMessage("Value "+IntToStr(handle)+" not found in "+LanguageFile+", using\
           default language label");
    label = GetStandartMsg(handle);
    return label;
  }
  label = ini->ReadString("Messages",IntToStr(handle),"");
  delete ini;
  return label;
}

void TForm1::ReadLabels() {
  if (LanguageFile!="") {
    TIniFile *ini = new TIniFile(LanguageFile);
    for (int i=0; L_handles[i]; i++) {
      if (!ini->ValueExists("Messages",IntToStr(L_handles[i]))) {
        ShowMessage("Incomplete language file "+LanguageFile+" ["+IntToStr(L_handles[i])+"]");
        LanguageFile = "";
        delete ini;
        ReadLabels();
        return;
      }
    }
    delete ini;
  }
  //Form1
  Button1->Caption = GetMsg(L_L_VOSS_SETUP);
//  Button9->Caption = GetMsg(L_L_EZ_SETUP);
  Button13->Caption = GetMsg(L_L_BROWSE_IMAGES);
  Button6->Caption = GetMsg(L_L_OPTIONS);
  BitBtn1->Hint = GetMsg(L_L_ABOUT_HINT);
  //Form3
  Form3->Caption = GetMsg(L_L_IMAGE_BROWSER);
  Form3->Label1->Caption = GetMsg(L_L_FORMATS);
  Form3->CheckBox1->Caption = GetMsg(L_L_CONST_SIZE);
  //Form4 (options)
  Form4->Caption = GetMsg(L_L_OPTIONS);
  Form4->Label3->Caption = GetMsg(L_L_VOSS_IMAGES);
  Form4->Label4->Caption = GetMsg(L_L_WATEC_IMAGES);
  Form4->CheckBox1->Caption = GetMsg(L_L_TRAY_MINIM);
  Form4->Label1->Caption = GetMsg(L_L_LANGUAGE);
  Form4->Button1->Caption = GetMsg(L_L_OK);
  Form4->Button2->Caption = GetMsg(L_L_CANCEL);
  Form4->Label2->Caption = GetMsg(L_L_SYNTAX_SETUP);
  Form4->Label5->Caption = "("+GetMsg(L_L_RESTART_CHANGES)+")";
  Form4->Label6->Caption = GetMsg(L_L_C_NORMAL)+":";
  Form4->Label7->Caption = GetMsg(L_L_C_WARNING)+":";
  Form4->Label8->Caption = GetMsg(L_L_C_ERROR)+":";
  Form4->Label9->Caption = GetMsg(L_L_C_SDIG)+":";
  Form4->Label10->Caption = GetMsg(L_L_EZ)+":";
  Form4->TabSheet1->Caption = GetMsg(L_L_DIRS);
  Form4->TabSheet2->Caption = GetMsg(L_L_COLORS);
  Form4->TabSheet3->Caption = GetMsg(L_L_SHORTCUTS);
    //shortcuts:
  Form4->Label11->Caption = GetMsg(L_L_SC_OPTIONS);
  Form4->Label12->Caption = GetMsg(L_L_SC_BROWSE);
  Form4->Label13->Caption = GetMsg(L_L_SC_VOSS);
  Form4->Label14->Caption = GetMsg(L_L_SC_EZ);
  //Form6
  Form6->Label1->Caption = GetMsg(L_L_VOSS_V_RES);
  Form6->Button1->Caption = GetMsg(L_L_OK);
  Form6->Button2->Caption = GetMsg(L_L_CANCEL);
  //About dlg
  AboutBox->Caption = GetMsg(L_L_ABOUT);
  AboutBox->Version->Caption = GetMsg(L_L_VER);
  AboutBox->TabSheet1->Caption = GetMsg(L_L_HELP);
  AboutBox->TabSheet2->Caption = GetMsg(L_L_HISTORY);
  AboutBox->TabSheet3->Caption = GetMsg(L_L_ABOUT_PR);
  AboutBox->Label3->Caption = GetMsg(L_L_THANKS_TO)+":";
    //naplneni aboutboxu:
    AboutBox->Memo1->Clear();
    AboutBox->Memo2->Clear();
    if (LanguageFile != "") {
      try {
        AboutBox->Memo1->Lines->LoadFromFile(ChangeFileExt(LanguageFile,".dok"));
      }
      catch (...) {
        AboutBox->Memo1->Lines->Add(GetMsg(L_L_F_NOT_FOUND)+
           " <"+ChangeFileExt(LanguageFile,".dok")+">");
        ShowMessage(GetMsg(L_L_CANT_OPEN)+" "+ChangeFileExt(LanguageFile,".dok"));
      }
      try {
        AboutBox->Memo2->Lines->LoadFromFile(ChangeFileExt(LanguageFile,".new"));
      }
      catch (...) {
        AboutBox->Memo2->Lines->Add(GetMsg(L_L_F_NOT_FOUND)+
           " <"+ChangeFileExt(LanguageFile,".new")+">");
        ShowMessage(GetMsg(L_L_CANT_OPEN)+" "+ChangeFileExt(LanguageFile,".new"));
      }
    }
  //Popup menu
  Obnovit1->Caption = GetMsg(L_L_POPUP_RESTORE);
  Ukonit1->Caption = GetMsg(L_L_POPUP_EXIT);
}

/************************
      EZ-Capture
*************************/

HWND HandleStinitka = 0;
int gDriverCount; //pocet driveru
bool DriverNapojen = false;
bool Snimam = false;  //priznak beziciho obrazu
CAPDRIVERCAPS gCapDriverCaps; //vlastnosti driveru
CAPSTATUS gCapStatus; //stav okna
bool OknoViditelne;
int EZx, EZy, EZw, EZh;

/*void EZinit() {
  DriverNapojen = false;
  Snimam = false;
  OknoViditelne = false;
  HandleStinitka = 0;
}

int TForm1::EZVytvorOkno(int x, int y, int w, int h) {
  if (HandleStinitka) return 0;
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_CREATE_WINDOW);
  HandleStinitka = capCreateCaptureWindow(WATECcaption,
    WS_OVERLAPPED|WS_POPUP|WS_BORDER|WS_DLGFRAME,
    x,y,w+capDX,h+capDY,Form1->Handle,0);
  if (!HandleStinitka) return 0;
  return 1;
}

int EZOkno() {
  if (!HandleStinitka) return 0;
  OknoViditelne = !OknoViditelne;
  if (OknoViditelne) ShowWindow(HandleStinitka,SW_SHOW);
  else ShowWindow(HandleStinitka,SW_HIDE);
  Form1->BringToFront();
  return 1;
}

int TForm1::EZPocetDriveru() {
  StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_ENUM_DRIVERS);
  char Verze[80];
  char VerzeDriveru[160];
  gDriverCount = 0;
  for (int i = 0; i < MAXVIDDRIVERS; i++) {
    if (capGetDriverDescription(i,(LPSTR)VerzeDriveru, sizeof(VerzeDriveru),
                                  (LPSTR)Verze, sizeof(Verze))) gDriverCount++;
  }
  return gDriverCount;
}

int TForm1::EZInicializaceHardwaru() {
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_CONNECT_DRIVER);
  if (!capDriverConnect(HandleStinitka, 0)) { //inicializuji vzdy driver #0
    DriverNapojen = false;
    return 0;
  }
  else DriverNapojen = true;
  Form1->msg(GetMsg(L_EZ_CONNECT_SUCCESS),0,c_ez);
  capDriverGetCaps(HandleStinitka,&gCapDriverCaps,sizeof(CAPDRIVERCAPS)); //nacist vlastnosti
  capGetStatus(HandleStinitka,&gCapStatus,sizeof(gCapStatus));
  capPreviewRate(HandleStinitka,vidPERIOD);
  return 1;
}

int EZStart_Stop_video() {
  if (!DriverNapojen) return 0;
  Snimam = !Snimam;
  capPreview(HandleStinitka,FALSE);
  capOverlay(HandleStinitka,(BOOL)Snimam);
  return 1;
}

int TForm1::EZstart() {
  EZinit();
  TIniFile *ini = new TIniFile( ChangeFileExt(Application->ExeName,".ini") );
  if (!EZVytvorOkno(ini->ReadInteger(EZcaption,"X",0),ini->ReadInteger(EZcaption,"Y",0),
                  ini->ReadInteger(EZcaption,"W",320),ini->ReadInteger(EZcaption,"H",240))) {
    Form1->msg(GetMsg(L_EZ_NO_WINDOW),1,c_error);
    return 0;
  }
  EZPocetDriveru();
  if (gDriverCount==0) {
    Form1->msg(GetMsg(L_EZ_NO_DRIVER),1,c_error);
    DestroyWindow(HandleStinitka);
    HandleStinitka = 0;
    return 0;
  }
  if (!EZInicializaceHardwaru()) {
    Form1->msg(GetMsg(L_EZ_NO_INIT),1,c_error);
    DestroyWindow(HandleStinitka);
    HandleStinitka = 0;
    return 0;
  }
  EZStart_Stop_video();
  return 1;
}

void EZgrabImage() {
  if (!HandleStinitka) return;
  capGrabFrameNoStop(HandleStinitka);
}

void TForm1::EZuloz(AnsiString jm) {
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_SAVE_TO)+jm+" . . .";
  if (!capFileSaveDIB(HandleStinitka,jm.c_str()))
    Form1->msg(GetMsg(L_EZ_CANT_SAVE)+jm,1,c_warning);
}

void TForm1::EZZavrit() {
  if (!DriverNapojen || !HandleStinitka) return;
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_CLOSE);
  Form1->msg(GetMsg(L_EZ_CLOSING),0,c_ez);
  RECT poloha;
  GetWindowRect(HandleStinitka,&poloha);
  int x = poloha.left, y = poloha.top, dx = poloha.right-poloha.left-capDX,
      dy = poloha.bottom-poloha.top-capDY;
  if (x<0 || x>Screen->Width) x = Screen->Width - 640;
  if (y<0 || y>Screen->Height) y = Screen->Height - 480;
  if (dx<0 || dx>640) dx = 640;
  if (dy<0 || dy>480) dy = 480;
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_WINDOW_POS);
  TIniFile *ini = new TIniFile( ChangeFileExt( Application->ExeName, ".ini") );
  ini->WriteInteger(EZcaption,"X",x);
  ini->WriteInteger(EZcaption,"Y",y);
  ini->WriteInteger(EZcaption,"W",dx);
  ini->WriteInteger(EZcaption,"H",dy);
  delete ini;
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_DISCONNECT);
  if (!capDriverDisconnect(HandleStinitka))
    Form1->msg(GetMsg(L_EZ_CANT_DISCONNECT),0,c_error);
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_EZ_CLOSE_WINDOW);
  if (!DestroyWindow(HandleStinitka))
    Form1->msg(GetMsg(L_EZ_CANT_DESTROY_WINDOW),0,c_error);
  Form1->msg(GetMsg(L_EZ_CLOSED),0,c_ez);
}
                                                                           */
/************************
          SDIG
*************************/

int TForm1::SetSizeTables() {
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_COMP_SCALE);
	if (height_table!=NULL) free(height_table);
	if ((height_table = (int*)malloc(MAXHEIGHT*sizeof(int)))==NULL) {
    Form1->msg(GetMsg(L_DG_NO_SCALE_MEMORY),1,c_error);
    Form1->StatusBar1->Panels->Items[1]->Text = "";
    return 0;
  }
  for (int i=0; i<HEIGHT; i++)
		height_table[i] = (int)((double)MAXHEIGHT/(double)HEIGHT * (double)i);
  Form1->StatusBar1->Panels->Items[1]->Text = "";
  return 1;
}

void DeleteSizeTables() {
  if (height_table!=NULL) free(height_table);
  height_table = NULL;
}

void __fastcall TForm1::msg(AnsiString txt, int show, TColor color) {
  if (!g_loging)
    if (show) ShowMessage(txt);
  AnsiString log = ActualPath + CAMLOG, logline;
  FILE *logf;
  if ((logf = fopen(log.c_str(),"at"))==NULL) {
    ShowMessage(GetMsg(L_DG_CANT_LOG) + log);
    g_loging = 0;
    return;
  }
  Word Year, Month, Day, Hour, Min, Sec, MSec;
  TDateTime cas = Now();
  DecodeTime(cas, Hour, Min, Sec, MSec);
  DecodeDate(cas, Year, Month, Day);
  logline = IntToStr(Day/10)+ IntToStr(Day%10) + "/" + IntToStr(Month/10) +
            IntToStr(Month%10) + "/" + IntToStr(Year/10) + IntToStr(Year%10) +
            " " + IntToStr(Hour/10) + IntToStr(Hour%10) + ":" + IntToStr(Min/10)
            + IntToStr(Min%10) + ":" + IntToStr(Sec/10) + IntToStr(Sec%10)+
            + "."+IntToStr(MSec/100)+" : " + txt;
  int start = Memo1->Text.Length();
  Memo1->SelStart = start;
  if (Visible) Memo1->SetFocus();
  Memo1->SelText = logline;
  Memo1->SelStart = start;
  Memo1->SelLength = Memo1->Text.Length()-start;
  Memo1->SelAttributes->Color = color;
  Memo1->Lines->Add("");
  logline = logline + '\n';
  fwrite(logline.c_str(),strlen(logline.c_str()),1,logf);
  fclose(logf);
  if (show) ShowMessage(txt);
}
             
void TForm1::SetGrayPalette(Graphics::TBitmap *btmp)
{
  btmp->Width = WIDTH;
  btmp->Height = HEIGHT;
  btmp->PixelFormat = pf8bit;
  typedef struct {
    TLogPalette lPal;
    TPaletteEntry dummy[256];
  } LogPal;
  LogPal SysPal;
  SysPal.lPal.palVersion = 0x300;
  SysPal.lPal.palNumEntries = 256;
  for (int i=0; i<256; i++)
  {
    SysPal.dummy[i].peRed = (Byte)i;
    SysPal.dummy[i].peBlue = (Byte)i;
    SysPal.dummy[i].peGreen = (Byte)i;
    SysPal.dummy[i].peFlags = 0;
  }
  btmp->Palette = CreatePalette(&SysPal.lPal);
}

int TForm1::InitDyn(DEV_T FAR* dev) {
  Form1->msg(GetMsg(L_DG_MEM_ALLOC),0,c_sdig);
  buffer = ((unsigned __int16 *)mvGetRowAdr(dev,0,0));
  bufferpitch = ((unsigned __int16 *)mvGetRowAdr(dev,0,1)) - buffer;
  Form1->msg(GetMsg(L_DG_BUFFERPITCH) + IntToStr(bufferpitch),0,c_sdig);
  return 1;
}

void DisposeInits()
{
  delete pBitmap;
}

DEV_T FAR* TForm1::InitDevice(char* inifile, int* result) {
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_OPENING);
  return OpenDevice(NULL,inifile,result);
}

short TForm1::DeviceClose(DEV_T FAR* dev) {
	if (dev == NULL) return -1;
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_CLOSING);
	return CloseDevice();
}

WORD TForm1::ReadError(DEV_T FAR* dev, char* cErrorMessage) {
	WORD ErrCode;
	ErrCode = GetErr(dev);
	if ( ErrCode != NO_ERR )
	{
		if (!IsWarning(dev) || !IsFatalError(dev))
			sprintf (cErrorMessage, GetMsg(L_DG_ERROR_STRING).c_str(), GetErrStr(dev));
	}
	else strcpy(cErrorMessage,"");
  ClearErr(dev);
  return ErrCode;
}

t_header* TForm1::new_header(int HEIGHT,unsigned int Year,unsigned int Month,unsigned int Day,
                unsigned int Hour,unsigned int Min,unsigned int Sec,unsigned int MSec,
                int compress) {
  t_header *hlav;
  if ((hlav = (t_header*)malloc(sizeof(t_header)))==NULL) {
    Form1->msg(GetMsg(L_DG_HEADER_NO_MEMORY),1,c_error);
    return NULL;
  }
  strcpy((char*)&hlav->tri,TRI);
  hlav->version = DIG_VERSION;
  hlav->compressed = (compress ? 'c' : ' ');
  hlav->compr_eng_ver = COMPRESS_ENGINE_VERSION;
  strcpy((char*)&hlav->copyright,COPYRIGHT);
  hlav->voss_width = WIDTH;
  //TODO: uvazit moznost ukladani v jinem rozliseni nez zobrazovani:
  hlav->voss_height = HEIGHT;
  hlav->year = Year;
  hlav->month = Month;
  hlav->day = Day;
  hlav->hour = Hour;
  hlav->minute = Min;
  hlav->second = Sec;
  hlav->ssecond = MSec;
  return hlav;
}

void SdigPanel(AnsiString txt) {
  Form1->StatusBar1->Panels->Items[1]->Text = txt;
}

void EzCapturePanel(AnsiString txt) {
  Form1->StatusBar2->Panels->Items[1]->Text = txt;
}

int WriteHead(FILE *fil, t_header *hlav) {
  try {
    fwrite(&hlav->tri,HEAD_LENGTH,1,fil);
    fwrite(&hlav->version,sizeof(hlav->version),1,fil);
    fwrite(&hlav->compressed,sizeof(char),1,fil);
    fwrite(&hlav->compr_eng_ver,sizeof(hlav->compr_eng_ver),1,fil);
    fwrite(&hlav->copyright,COPY_LENGTH,1,fil);
    fwrite(&hlav->voss_width,sizeof(hlav->voss_width),1,fil);
    fwrite(&hlav->voss_height,sizeof(hlav->voss_height),1,fil);
    fwrite(&hlav->year,sizeof(signed int),1,fil);
    fwrite(&hlav->month,sizeof(signed int),1,fil);
    fwrite(&hlav->day,sizeof(signed int),1,fil);
    fwrite(&hlav->hour,sizeof(signed int),1,fil);
    fwrite(&hlav->minute,sizeof(signed int),1,fil);
    fwrite(&hlav->second,sizeof(signed int),1,fil);
    fwrite(&hlav->ssecond,sizeof(signed int),1,fil);
  }
  catch (...){
    return 0;
  }
  return 1;
}

void TForm1::DispShow(DEV_T FAR *dev, AnsiString *kamukl, TDateTime cas) {
  Byte *pt; //pointer na radek pBitmap
  unsigned __int16 *p, cisl;  //radek bufferu, jeden pixel
  int y, ukladej = 1;
  //Konverze data a casu:
  Word Year, Month, Day, Hour, Min, Sec, MSec;
  unsigned int iYear, iMonth, iDay, iHour, iMin, iSec, iMSec;
  DecodeTime(cas, Hour, Min, Sec, MSec); DecodeDate(cas,Year,Month,Day);
  iYear = (unsigned int)Year; iMonth = (unsigned int)Month;
  iDay = (unsigned int)Day; iHour = (unsigned int)Hour;
  iMin = (unsigned int)Min; iSec = (unsigned int)Sec; iMSec = (unsigned int)MSec;
  Form2->Caption = (AnsiString)SDIGcaption + " - " + IntToStr(iHour) + ":" +
                    IntToStr(iMin) + ":" + IntToStr(iSec) + "." + IntToStr(iMSec/100);
  //end of - Konverze data a casu
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_DRAWING);
  FILE *fil;
  if (kamukl!=NULL){
    SetCurrentDir(ExtractFilePath(*kamukl));
    if ((fil = fopen(ExtractFileName(*kamukl).c_str(),"wb"))==NULL) {
      Form1->msg(GetMsg(L_DG_CANT_OPEN_FILE) + *kamukl + ", "+GetMsg(L_DG_PICTURE_NOT_SAVED),1,c_warning);
      ukladej = 0;
    }
    else {  //hlavicka souboru
      t_header *hlavicka = new_header(HEIGHT,iYear,iMonth,iDay,iHour,
                                        iMin,iSec,iMSec,0);
      if (hlavicka == NULL) return;
      if (!WriteHead(fil,hlavicka)) {
        Form1->msg(GetMsg(L_DG_CANT_WRITE) +  *kamukl + ", "+GetMsg(L_DG_DISC_FULL),1,c_warning);
        fclose(fil);
        DeleteFile(*kamukl);
        ukladej = 0;
      }
      delete hlavicka;
    }
  }
  p = (unsigned __int16*)buffer;
  pt = (Byte*)pBitmap->ScanLine[0];
  for (int i=0; i<WIDTH; i++) { //pro kazdy pixel treba ztransformovat
    cisl = p[i];
    EVAL(cisl);
    pt[i] = (Byte)(cisl);
  }
  for ( p += bufferpitch * (height_table[1]-height_table[0]), y = 1;
        y < HEIGHT; y++, p += bufferpitch*(height_table[y]-height_table[y-1])) {
    pt = (Byte*)pBitmap->ScanLine[y];
    for (int i=0; i<WIDTH; i++){
      cisl = p[i];
      EVAL(cisl);
      pt[i] = (Byte)(cisl);
    }
  }
  Form2->Canvas->Draw(0,0,pBitmap);
  //ukladani:
  p = (unsigned __int16*)buffer;
  if (kamukl!=NULL && ukladej){
    Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_SAVING_TO)+(AnsiString)*kamukl+" . . .";
    if (fwrite(p,sizeof(unsigned __int16)*WIDTH,1,fil)!=1) {
      Form1->msg(GetMsg(L_DG_CANT_WRITE) +  *kamukl + ", "+GetMsg(L_DG_DISC_FULL),1,c_warning);
      fclose(fil);
      DeleteFile(*kamukl);
      ukladej = 0;
    }
    for ( p += bufferpitch * (height_table[1]-height_table[0]), y = 1;
          y < HEIGHT; y++, p += bufferpitch*(height_table[y]-height_table[y-1]))
      if (fwrite(p,sizeof(unsigned __int16)*WIDTH,1,fil)!=1) {
        Form1->msg(GetMsg(L_DG_CANT_WRITE) +  *kamukl + ", "+GetMsg(L_DG_DISC_FULL),1,c_warning);
        fclose(fil);
        DeleteFile(*kamukl);
        ukladej = 0;
      }
    fclose(fil);
  }
  //TODO: co tohle??????:
  SetCurrentDir(ActualPath);
}

int TForm1::SetImageHeight(int dy) {
  if (dy<=0) {
    Form1->msg(GetMsg(L_DG_V_RESOLUTION),1,c_error);
    return 0;
  }
  Form1->msg(GetMsg(L_DG_V_RES_CHANGE) + IntToStr(dy),0,c_sdig);
  HEIGHT = dy;
  Form2->Width = WIDTH + 7; Form2->Height = HEIGHT + 26;
  Form2->Image1->Height = HEIGHT;
  Form2->Image1->Width = WIDTH;
  pBitmap->Height = HEIGHT;
  pBitmap->Width = WIDTH;
  return SetSizeTables();
}

AnsiString TForm1::ReadLanguageFile(AnsiString lan) {
  if (lan == "") return "";
  TIniFile *ini = new TIniFile(ExtractFilePath(Application->ExeName)+"langcode.dat");
  if (ini->ReadString("Header","Header","") != L_CODE_DAT) {
    ShowMessage("Invalid language list file "+ini->FileName);
    delete ini;
    return "";
  }
  if (!ini->ValueExists("Languages",lan)) {
    delete ini;
    ShowMessage("Invalid language: "+lan);
    return "";
  }
  lan = ExtractFilePath(Application->ExeName)+lan+".lng";
  if (!FileExists(lan)) {
    delete ini;
    ShowMessage("Language file "+lan+" does not exist");
    return "";
  }
  delete ini;
  //test verse souboru s jazykem
  ini = new TIniFile(lan);
  AnsiString jazyk = ini->ReadString("Header","Language","");
  if (jazyk != ChangeFileExt(ExtractFileName(lan),"")) {
    delete ini;
    ShowMessage("Language not correct");
    return "";
  }
  if (ini->ReadInteger("Header","MajorVersion",-1)!=MAJOR_VER ||
      ini->ReadInteger("Header","MinorVersion",-1)!=MINOR_VER) {
    delete ini;
    ShowMessage("Language file version error");
    return "";
  }
  delete ini;
  return lan;
}

void __fastcall TForm1::FormCreate(TObject *Sender)
{
  TIniFile *ini;
  ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
  LanguageFile = ReadLanguageFile(ini->ReadString("Global","Language",""));
  Top = ini->ReadInteger( "MainForm", "Y", 0 );
  Left = ini->ReadInteger( "MainForm", "X", 0 );
  HEIGHT = ini->ReadInteger(SDIGcaption, "dispY", 512);
  SDIGdir = ini->ReadString(SDIGcaption, "dirS", ExtractFilePath(Application->ExeName));
//  EZdir = ini->ReadString(EZcaption,"dirS",ExtractFilePath(Application->ExeName));
  c_normal=(TColor)ini->ReadInteger("Global","c_normal",C_NORMAL);
  c_warning=(TColor)ini->ReadInteger("Global","c_warning",C_WARNING);
  c_error=(TColor)ini->ReadInteger("Global","c_error",C_ERROR);
  c_sdig=(TColor)ini->ReadInteger("Global","c_sdig",C_SDIG);
//  c_ez=(TColor)ini->ReadInteger("Global","c_ez",C_EZ);
  //klavesove zkratky
  AkNastaveni->ShortCut = ini->ReadInteger("Shortcuts","Options",0);
  AkProhlizet->ShortCut = ini->ReadInteger("Shortcuts","Browse images",0);
  AkVossSetup->ShortCut = ini->ReadInteger("Shortcuts","Voss setup",0);
//  AkEZsetup->ShortCut = ini->ReadInteger("Shortcuts","EZ setup",0);
  //logovani
  g_loging = 1;
  if (FileExists(ActualPath+CAMLOG))
    if (!DeleteFile(ActualPath+CAMLOG))
     ShowMessage(GetMsg(L_DG_CANT_DELETE) + ActualPath+CAMLOG);
  msg(GetMsg(L_L_START_LOGGING),0,c_normal);
  delete ini;
  ActualPath = ExtractFilePath(Application->ExeName);
  SetCurrentDir(ActualPath);
  g_exit_win = 0;
  StatusBar1->Panels->Items[0]->Text = SDIGcaption;
  StatusBar2->Panels->Items[0]->Text = EZcaption;
  pBitmap = new Graphics::TBitmap();  //bitmapa pro kopirovani SDIG dat
  if (pBitmap==NULL) {
    msg(GetMsg(L_DG_PBITMAP_ERROR),1,c_error);
    return;
  }
  SetGrayPalette(pBitmap);
  Form2->Caption = SDIGcaption;
}

void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
  TIniFile *ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
//  ini->WriteBool("Global","SysTray",SysTray);
  if (LanguageFile=="") ini->WriteString("Global","Language","");
  ini->WriteInteger("MainForm", "X", Left);
  ini->WriteInteger("MainForm", "Y", Top);
  ini->WriteInteger("SDIGform", "X", Form2->Left);
  ini->WriteInteger("SDIGform", "Y", Form2->Top);
  //barvy syntaxe
  ini->WriteInteger("Global","c_normal",c_normal);
  ini->WriteInteger("Global","c_warning",c_warning);
  ini->WriteInteger("Global","c_error",c_error);
  ini->WriteInteger("Global","c_sdig",c_sdig);
  ini->WriteInteger("Global","c_ez",c_ez);
  ini->WriteInteger("Shortcuts","Options",AkNastaveni->ShortCut);
  ini->WriteInteger("Shortcuts","Browse images",AkProhlizet->ShortCut);
  ini->WriteInteger("Shortcuts","Voss setup",AkVossSetup->ShortCut);
  ini->WriteInteger("Shortcuts","EZ setup",AkEZsetup->ShortCut);
  delete ini;
//  if (DriverNapojen || HandleStinitka) EZZavrit();
  DeleteSizeTables();
  Action = caFree;
  g_konec = 1;
  g_pr = 0;
}

void __fastcall TForm1::Button6Click(TObject *Sender)
{
  try {
    Form4->DirectoryListBox1->Directory = SDIGdir;
//    Form4->DirectoryListBox2->Directory = EZdir;
  }
  catch (...) {
    Form4->DirectoryListBox1->Directory = ExtractFilePath(Application->ExeName);
    Form4->DirectoryListBox2->Directory = ExtractFilePath(Application->ExeName);
  }
  AnsiString str;
  //klaves. zkratky:
  Form4->HotKey1->HotKey = AkNastaveni->ShortCut;
  Form4->HotKey2->HotKey = AkProhlizet->ShortCut;
  Form4->HotKey3->HotKey = AkVossSetup->ShortCut;
  Form4->HotKey4->HotKey = AkEZsetup->ShortCut;
  //dialog...:
  if (Form4->ShowModal()!=mrOk) return;
  c_normal = Form4->Image1->Canvas->Brush->Color;
  c_warning = Form4->Image2->Canvas->Brush->Color;
  c_error = Form4->Image3->Canvas->Brush->Color;
  c_sdig = Form4->Image4->Canvas->Brush->Color;
  c_ez = Form4->Image5->Canvas->Brush->Color;
  AnsiString dir_sdig, dir_ez;
  dir_sdig = Form4->DirectoryListBox1->Directory;
  dir_ez = Form4->DirectoryListBox2->Directory;
  SDIGdir = dir_sdig;
//  EZdir = dir_ez;
  TIniFile *ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
  ini->WriteString(SDIGcaption, "dirS", SDIGdir);
//  ini->WriteString(EZcaption,"dirS",EZdir);
  if (Form4->Combo->ItemIndex!=-1) {
    AnsiString lan = Form4->Combo->Items->Strings[Form4->Combo->ItemIndex];
    int strt = lan.Pos("["), end = lan.Pos("]");
    lan = lan.SubString(strt+1,end-strt-1);
    ini->WriteString("Global","Language",lan);
  }
  if (ReadLanguageFile(ini->ReadString("Global","Language","")) != LanguageFile) {
    LanguageFile = ReadLanguageFile(ini->ReadString("Global","Language",""));
    ReadLabels();
  }
  // klavesove zkratky
  AkNastaveni->ShortCut = Form4->HotKey1->HotKey;
  AkProhlizet->ShortCut = Form4->HotKey2->HotKey;
  AkVossSetup->ShortCut = Form4->HotKey3->HotKey;
//  AkEZsetup->ShortCut = Form4->HotKey4->HotKey;
  delete ini;
}

void invbit(unsigned int &x, int i)
{
   DWORD k = 1;
   for (int l=1; l<i; k *= 2, l++);
   x ^= k;
}

int TForm1::SDIGclose(DEV_T FAR *dev) {
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_CLOSING);
  Form1->msg(GetMsg(L_DG_CLOSING_SDIG),0,c_sdig);
  unsigned int x;
  x = mvReadDigIO(dev);
  invbit(x,6);
  mvWriteDigIO(dev,x);
  DeleteSizeTables();
  if (!DeviceClose(dev)) Form1->msg(GetMsg(L_DG_DEVICE_NOT_CLOSED),1,c_warning);
  else Form1->msg(GetMsg(L_DG_DEVICE_CLOSED),0,c_sdig);
  DisposeInits();
  return 1;
}

void TForm1::whc() {
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_WAITING_SYNC);
  Form1->msg(GetMsg(L_DG_WAIT_SYNCHR),0,c_normal);
  AnsiString jm = ExtractFilePath(Application->ExeName)+ININAME, tm, cs, csez;
  int sn = 0, sek = 0, konec = 0;
  TDateTime cas;
  Word Year, Month, Day, Hour, Min, Sec, MSec;
  TIniFile *fil = new TIniFile(jm);
  while (g_pr && !g_konec) {
    if (FileExists(jm)) {
        Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_NEW_COMMAND);
        Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_NEW_COMMAND);
        fil->UpdateFile();
        sn = fil->ReadInteger(SECTION,INI_SINGLE,0);
        sek = fil->ReadInteger(SECTION,INI_SEQUENCE,0);
        konec = fil->ReadInteger(SECTION,INI_END,0);
		    AnsiString txt =  GetMsg(L_DG_COMMAND) + INI_SINGLE+" = " + IntToStr(sn) + ", "+
                          INI_SEQUENCE+" = "+ IntToStr(sek)+", "+ INI_END+" = "
                          +IntToStr(konec);
        Form1->msg(txt,0,c_normal);
        while (!DeleteFile(jm)){};
    }
    if (konec) {
      Form1->Button13->Enabled = false;
      g_pr = 0;
      g_exit_win = 1;
      Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_HALT);
      Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_HALT);
    }
    else if (sn && !sek) {
      Form1->Button13->Enabled = true;
      Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_SAVING_SINGLE);
      Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_SAVING_SINGLE);
      sn = sek = 0;
      mvSingleSnap(dev,1);
//      EZgrabImage();
      cas = Now();
      DecodeTime(cas, Hour, Min, Sec, MSec);
      DecodeDate(cas,Year,Month,Day);
      cs = IntToStr((Year/10)%10)+IntToStr(Year%10)+"_"+IntToStr(Month/10)+IntToStr(Month%10)+
           "_"+IntToStr(Day/10)+IntToStr(Day%10)+"_"+IntToStr(Hour/10)+IntToStr(Hour%10)+"_"+
           IntToStr(Min/10)+IntToStr(Min%10)+"_"+IntToStr(Sec/10)+IntToStr(Sec%10)+
           "."+IntToStr(MSec/100);
      csez = "s_ezcp_"+cs;
      cs   = "s_voss_"+cs;
      cs   = SDIGdir+"\\"+cs+".dig";
//      csez = EZdir+"\\"+csez+".bmp";
      Application->ProcessMessages();
//      EZStart_Stop_video();
      DispShow(dev,&cs,cas);
/*      EZuloz(csez);
      EZStart_Stop_video();*/
    }
    else if (sek && !sn) {
      Form1->Button13->Enabled = false;
      Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_SAVING_SEQ);
      Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_SAVING_SEQ);
      mvSingleSnap(dev,1);
//      EZgrabImage();
      cas = Now();
      DecodeTime(cas, Hour, Min, Sec, MSec);
      DecodeDate(cas,Year,Month,Day);
      cs = IntToStr((Year/10)%10)+IntToStr(Year%10)+"_"+IntToStr(Month/10)+IntToStr(Month%10)+
           "_"+IntToStr(Day/10)+IntToStr(Day%10)+"_"+IntToStr(Hour/10)+IntToStr(Hour%10)+"_"+
           IntToStr(Min/10)+IntToStr(Min%10)+"_"+IntToStr(Sec/10)+IntToStr(Sec%10)+
           "."+IntToStr(MSec/100);
//      csez = "c_ezcp_"+cs;
      cs   = "c_voss_"+cs;
      cs   = SDIGdir+"\\"+cs+".dig";
//      csez = EZdir+"\\"+csez+".bmp";
      Application->ProcessMessages();
//      EZStart_Stop_video();
      DispShow(dev,&cs,cas);
/*      EZuloz(csez);
      EZStart_Stop_video();*/
    }
    else if (sek && sn || !sn && !sek) {
      cas = Now();
      Form1->Button13->Enabled = true;
      Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_NOT_SAVE);
      Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_NOT_SAVE);
      sek = sn = 0;
      mvSingleSnap(dev,1);
      DispShow(dev,NULL,cas);
    }
    Application->ProcessMessages();
  }
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_STOP);
  Form1->StatusBar2->Panels->Items[1]->Text = GetMsg(L_DG_STOP);
  delete fil;
}

int TForm1::Turn2IOD(DEV_T FAR *dev) {
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_IOD);
  Form1->msg(GetMsg(L_DG_SET_IOD),0,c_sdig);
  unsigned int x;
  mvInitDigIO(dev,3);
  x = mvReadDigIO(dev);
  invbit(x,6);
  mvWriteDigIO(dev,x);
  return 1;
}

int TForm1::CamerasStart() {
	int result;
	char  ErrorString[255];
  Form1->msg(GetMsg(L_DG_OPENING_DEV),0,c_sdig);
	dev = InitDevice (".\\sdig.ini", &result);
  if (dev == NULL) {
    Form1->msg(GetMsg(L_DG_NOT_OPEND) + IntToStr(result),1,c_error);
    return 0;
  }
  Form1->msg(GetMsg(L_DG_SUCCESS),0,c_sdig);
  ReadError(dev, ErrorString);
  if (strcmp(ErrorString,"")) {
    Form1->msg(GetMsg(L_DG_SENT_ERROR) + (AnsiString)ErrorString,1,c_error);
    return 0;
  }
  if (!InitDyn(dev)) return 0;
  if (!SetImageHeight(HEIGHT)) return 0;
  if (!Turn2IOD(dev)) return 0;
  if (!Form2->Visible) Form1->StatusBar1Click(NULL);
  g_pr = 1;
//  if (!EZstart()) return 0;
  Form1->StatusBar2Click(NULL);
  whc();
  if (!SDIGclose(dev)) return 0;
  return 1;
}

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
  Timer1->Enabled = false;
  SetImageHeight(HEIGHT);
  AnsiString jm = ExtractFilePath(Application->ExeName)+ININAME;
  g_konec = 0;
  msg(GetMsg(L_DG_WAIT_FOR_SYNC),0,c_normal);
  while (!FileExists(jm) && !g_konec) Application->ProcessMessages();

  TIniFile *ini = new TIniFile(ExtractFilePath(Application->ExeName)+ININAME);
  if (ini->ReadInteger(SECTION,INI_END,0)) {
    msg(GetMsg(L_DG_NOT_INIT),0,c_error);
    DeleteFile(ExtractFilePath(Application->ExeName)+ININAME);
    Close();
  }

  if (g_konec) {
    msg(GetMsg(L_DG_EXIT_CLEAR),0,c_normal);
    msg(GetMsg(L_L_END_LOGGING),0,c_normal);
    return;
  }
  msg(GetMsg(L_DG_INIT),0,c_normal);
  while (!DeleteFile(jm)) {};
  if (!CamerasStart()) { //start expozic
    msg(GetMsg(L_DG_NOT_INIT),0,c_error);
    msg(GetMsg(L_L_END_LOGGING),0,c_normal);
    return;
  }
  if (g_exit_win) {
    msg(GetMsg(L_DG_SYSTEM_HALT),0,c_normal);
    TCloseAction act = caNone;
    FormClose(Sender, act);
    msg(GetMsg(L_L_END_LOGGING),0,c_normal);
    ExitWindows(EWX_SHUTDOWN,0);
  }
  msg(GetMsg(L_L_END_LOGGING),0,c_normal);
}

void __fastcall TForm1::Button13Click(TObject *Sender)
{
  if (HandleStinitka!=0) capGrabFrameNoStop(HandleStinitka);
//  EZStart_Stop_video();
  Form3->ShowModal();
//  EZStart_Stop_video();
}

void __fastcall TForm1::Button1Click(TObject *Sender)
{
  Form1->StatusBar1->Panels->Items[1]->Text = GetMsg(L_DG_SETUP);
  Form6->UpDown1->Position = HEIGHT;
  Form6->Edit1->Text = IntToStr(HEIGHT);
  Form1->StatusBar1->Panels->Items[1]->Text = "";
  if (Form6->ShowModal()!=mrOk) {
    Form1->StatusBar1->Panels->Items[1]->Text = "";
    return;
  }
  Form1->StatusBar1->Panels->Items[1]->Text = "";
  int height;
  try {
    height = StrToInt(Form6->Edit1->Text);
  }
  catch (...) {
    ShowMessage(GetMsg(L_DG_ERROR)+Form6->Edit1->Text+" "+GetMsg(L_DG_NOT_INTEGER));
    return;
  }
  if (height<=0 || height>1024) {
    ShowMessage(GetMsg(L_DG_INTERVAL));
    return;
  }
  if (HEIGHT != height) {
    HEIGHT = height;
    SetImageHeight(HEIGHT);
  }
  TIniFile *ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
  ini->WriteInteger(SDIGcaption, "dispY", HEIGHT);
  delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::StatusBar2Click(TObject *Sender)
{
  if (!HandleStinitka) return;
//  EZOkno();
  if (OknoViditelne) StatusBar2->Panels->Items[0]->Bevel = pbLowered;
  else StatusBar2->Panels->Items[0]->Bevel = pbRaised;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::StatusBar1Click(TObject *Sender)
{
  Form2->Visible = !Form2->Visible;
  if (Form2->Visible) StatusBar1->Panels->Items[0]->Bevel = pbLowered;
  else StatusBar1->Panels->Items[0]->Bevel = pbRaised;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::BitBtn1Click(TObject *Sender)
{
  AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Timer2Timer(TObject *Sender)
{
  TDateTime cas;
  Word Year, Month, Day, Hour, Min, Sec, MSec;
  cas = Now();
  DecodeTime(cas, Hour, Min, Sec, MSec); DecodeDate(cas,Year,Month,Day);
  Label1->Caption = IntToStr(Hour/10)+IntToStr(Hour%10)+":"+
                    IntToStr(Min/10)+IntToStr(Min%10)+":"+
                    IntToStr(Sec/10)+IntToStr(Sec%10);
  Label2->Caption = IntToStr(Day/10)+IntToStr(Day%10)+"."+
                    IntToStr(Month/10)+IntToStr(Month%10)+"."+
                    IntToStr(Year);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Obnovit1Click(TObject *Sender)
{
  Update();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Ukonit1Click(TObject *Sender)
{
  Close();  
}
//---------------------------------------------------------------------------

// Klavesove zkratky:

void __fastcall TForm1::AkNastaveniExecute(TObject *Sender)
{
  Button6->Click();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AkProhlizetExecute(TObject *Sender)
{
  Button13->Click();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AkVossSetupExecute(TObject *Sender)
{
  Button1->Click();
}
//---------------------------------------------------------------------------

/*void __fastcall TForm1::AkEZsetupExecute(TObject *Sender)
{
//  Button9->Click();
} */
//---------------------------------------------------------------------------

void __fastcall TForm1::Button2Click(TObject *Sender)
{
  if (FormDigfit->ShowModal()!=mrOk) return;
  AnsiString dir = FormDigfit->List->Directory;
  int fc = FormDigfit->FileList->Items->Count;
  for (int i=0; i<fc; i++)
  {
     AnsiString fn = FormDigfit->FileList->Items->Strings[i];
     convert_digfit(fn);
  }

}
//---------------------------------------------------------------------------

