#include <stdio.h>
#include "include\mv.h"
#include "include\mody.h"

#define UNUSED(v)  if(v) v=v

static int m_xVideoOffs = 0;
static int m_yVideoOffs = 0;

static LPCAMTYPE_T pActCamType      = NULL;

/* forward declaration */
void UpdateVideoTimingRam (DEV_T FAR* dev, int xSize, int ySize);


void ChangeColorMode (DEV_T FAR* dev, WORD cMode)
{
	/* SetColorMode */
	mvSelColorMode (dev, cMode);

	return;
}


void ChangeVideoMode (DEV_T FAR* dev, WORD vMode)
{
	/* SetVideoMode */
	mvSelVideoMode (dev, vMode);

	return;
}


void ChangeImagePos (DEV_T FAR* dev,  int xPos, int yPos)
{
	
	ACQUIRE_DEF_T AcqDef;
	int yposdif;
	
	/* fill ACQUIRE_DEF_T-struct with values from DEV_T-Struct */
	mvGetAcquireDef ( dev, &AcqDef);


	/* set x-offset for both fields */
	AcqDef.xpos[0] = xPos;
	AcqDef.xpos[1] = xPos;

	/* set y-offset for both fields */
	yposdif = AcqDef.ypos[1] - AcqDef.ypos[0];

	if (yposdif > 0)
	{
		AcqDef.ypos[0] = yPos;
		AcqDef.ypos[1] = yPos + yposdif;
	}

	if (yposdif < 0)
	{
		AcqDef.ypos[0] = yPos - yposdif;
		AcqDef.ypos[1] = yPos;
	}


	/* validate new values */
	mvSetAcquireDef ( dev, &AcqDef);
}


void ChangeImageSize (DEV_T FAR* dev, int xSize, int ySize)
{
	
	ACQUIRE_DEF_T AcqDef;
	
	/* fill ACQUIRE_DEF_T-struct with values from DEV_T-Struct */
	mvGetAcquireDef ( dev, &AcqDef);

	/* set softlive */
	AcqDef.start_addr[0] = 0;
	AcqDef.start_addr[1] = 0;


	/* set xSize for both fields */
	AcqDef.xsize[0] = xSize;
	AcqDef.xsize[1] = xSize;

	if(IsInterlaced (dev) == 0) //noninterlaced
	{
		/* set ySize */
		AcqDef.ysize[0] = ySize;
		AcqDef.ysize[1] = ySize; // not necessary
	}
	else   // interlaced
	{
		/* set ySize for both fields */
		AcqDef.ysize[0] = ySize/2;
		AcqDef.ysize[1] = ySize/2;
	}

	/* validate new values */
	mvSetAcquireDef ( dev, &AcqDef);

	UpdateVideoTimingRam (dev, xSize, ySize); // updates video timing ram

	return;
}

/* ------------------------------ mvGetActCameraTypePtr ----------------- */

LPCAMTYPE_T  _MVDFKT mvGetActCameraTypePtr (DEV_T FAR* dev)
{
   UNUSED (dev);
   return pActCamType;
}

/* is signal interlaced? */
int IsInterlaced (DEV_T FAR* dev)
{
  LPCAMTYPE_T		pCam;

  pCam = NULL;
  pCam = mvGetActCameraTypePtr( dev );

  if (pCam == NULL) // no camdefs used
   return 1;
  else
    return pCam->Interlaced ;
}


void ChangeVideoStartPos( DEV_T FAR* dev, int xVideoOffs, int yVideoOffs )
{
	ACQUIRE_DEF_T acq;
	int xSize, ySize;

	m_xVideoOffs = xVideoOffs;
	m_yVideoOffs = yVideoOffs;

	// get image size
	mvGetAcquireDef(dev,&acq);
	xSize = acq.xsize[0];
	ySize = acq.ysize[0] * acq.fields_per_frame;

	UpdateVideoTimingRam(dev, xSize, ySize );
}

/* returns PixelSize depending on Colormode */
int GetBitPerPixel (DEV_T FAR* dev)
{

      switch ( mvSelColorMode ( dev, NO_CHANGE )  ) 
      {
         case COL_GREY :
         case COL_GREY2 :
         case COL_CHROM :
         case COL_RED :
         case COL_GREEN :
         case COL_BLUE :
         case COL_RGB332 :
         case COL_GREY8 :
         case COL_YC_PK :
         case COL_YC_PL :
            return 8;
            break ;
         case COL_RGB555 :
            return 15;
            break;
         case COL_RGB565 :
            return 16;
            break ;
         case COL_RGB24 :
            return 24;
            break ;
         case COL_RGB888 :
         case COL_BGR888 :
            return 32;
            break ;
         case COL_RGB32 :
            return 32;
            break ;
         case COL_CLASSCOLO :
            return 15;
            break ;
         case COL_CLASSRESO :
            return 8;
            break ;
         default :
			return 8;
            break ;
	  }
	  return 0;
}


// uf:18.12.98 ungetestete nderung bzgl. Ergnzung x,y Verschiebung

void UpdateVideoTimingRam (DEV_T FAR* dev, int xSize, int ySize)
{

    LPCAMTYPE_T		pCam;
    WORD		    xStart, yStart;

	pCam = NULL;

	pCam = mvGetActCameraTypePtr( dev );

	if (pCam == NULL) // no camdef used
	{
		//defaultwerte:
		xStart = 50;
		yStart = 19;
	}
	else			// camdef used
	{
		//use camdefwerte
		xStart = (WORD)pCam->HTAcquireStart;
		yStart = (WORD)pCam->VTAcquireStart;
	}

	
    xStart+=m_xVideoOffs;
	if (IsInterlaced(dev)==0) // non interlaced
	   yStart+=m_yVideoOffs;
	else
	   yStart+=m_yVideoOffs/2;




    mvSetVideoTiming (dev, HRAM, 0, 4095, 1, HVT_ACQUIRE, BIT_RESET);
    mvSetVideoTiming (dev, HRAM, xStart, (WORD) (xStart + (xSize - 1)), 1, HVT_ACQUIRE, BIT_SET);

	if (IsInterlaced(dev)==0) // non interlaced
	{
		mvSetVideoTiming (dev, VRAM, 0, 4095, 1, HVT_ACQUIRE, BIT_RESET);
		mvSetVideoTiming (dev, VRAM, yStart ,(WORD) (yStart + ySize -1 ) , 1, HVT_ACQUIRE, BIT_SET);
	}
	else     // interlaced
	{
		mvSetVideoTiming (dev, VRAM, 0, 4095, 1, HVT_ACQUIRE, BIT_RESET);
		mvSetVideoTiming (dev, VRAM, yStart ,(WORD) (yStart + (ySize / 2)-1) , 1, HVT_ACQUIRE, BIT_SET);
	}

    mvLoadTimingRAM (dev);
}
