#define CT_NAME_LENGTH  40

typedef struct property_data_s
{
    DWORD count;                  /* number of properties in the data block       */
    char  FAR* FAR* names;        /* pointer to a string list with property names */
    DWORD FAR* data;              /* memory holder for property data              */
} CAMPROP_T, FAR* LPCAMPROP_T;

typedef struct camtype_t
{
    /* CAMERA DATA */
    char Name[CT_NAME_LENGTH];   /* name of the camera                             */
    WORD VideoMode;              /*                                                */
    WORD Interlaced;             /* number of fields per frame                     */
    WORD VerticalFreq;           /* Vertical scan frequency in Hz                  */
    WORD HorizontalFreq;         /* Horizontal scan frequency in Hz                */
    WORD ClockFreq;              /* Clock frequency in kHz                         */
    WORD ClockMode;              /* 0: internal; 1: external                       */

    /* ACQUIRE SETUP */
    WORD AcquireMode;            /* 0: Composite Sync; 1: Variable Scan            */
    WORD InvertSync;             /* Bit 0: pixelclock invert;
                                    Bit 1: line sync invert;
                                    Bit 2: frame sync invert;                      */
    WORD StartMode;              /* 1: Odd; 2: Even; 3: Next                       */
    WORD ChrominanceFilter;      /* 0: AC coupled; 1: 50Hz; 2: 60Hz; 3: DC coupled */
    WORD Gain[32];               /* 0: 0.5; 1: 1; 2: 2; 3: 4;                      */
    INT  Offset[32];             /* in mV                                          */
    WORD LowReference[32];       /* in mV                                          */
    WORD HighReference[32];      /* in mV                                          */

    /* TIMING RAM SETUP */
    WORD HTParameterMode;        /* horizontal parameter 0: in pixel 1: in ns;     */
    DWORD HTAcquireStart;        /* acquire start position                         */
    DWORD HTAcquireWidth;        /* acquire width                                  */
    WORD HTAcquireIncrement;     /* acquire increment                              */

    DWORD HTClampStart;          /* clamp start position                           */
    DWORD HTClampWidth;          /* clamp width 0: set to default 5                */
    DWORD HTZeroStart;           /* zero start position                            */
    DWORD HTZeroWidth;           /* zero width 0: set to default 5                 */
    DWORD HTFieldStart;          /* field gate start position 0: auto              */
    DWORD HTFieldWidth;          /* field gate width 0: auto                       */

    WORD  VTParameterMode;       /* vertical parameter 0: in lines 1: in us;       */
    DWORD VTAcquireStart;        /* acquire start position                         */
    DWORD VTAcquireWidth;        /* acquire width                                  */
    WORD  VTAcquireIncrement;    /* acquire increment                              */

    WORD  GenerateVSync;         /* 0 : no; 1 : yes                                */
    DWORD GenerateVSyncStart;    /* Startposition of vsync                         */
    DWORD GenerateVSyncWidth;    /* Width of Sync pulse                            */
    WORD  GenerateVSyncPol;      /* Sync polarity: 0:Sync is low; 1:Sync is high   */

    /* new elements from 9.9.99 */
    DWORD HTAcquireTotal;        /* time or pixel between two hsync's              */
    WORD  PhysCamType;           /* bit0: digital, bit1: linescan                  */
    char *Manufacturer;          /* name of the camera manufactor                  */
    char *PropDlgDLL;            /* DLL filename with properties dialog            */
    char *PropDlgFuncName;       /* DLL function name of property dialog           */
    CAMPROP_T PropData;          /* Property data, camera dependent                */
    char *Comment;               /* additional information text                    */

} CAMTYPE_T, FAR* LPCAMTYPE_T;


int IsInterlaced (DEV_T FAR* dev);
void ChangeVideoMode (DEV_T FAR*, WORD);
void ChangeColorMode (DEV_T FAR*, WORD);
void ChangeImageSize (DEV_T FAR*, int, int);
void ChangeImagePos  (DEV_T FAR*, int, int);
void ChangeVideoStartPos(DEV_T FAR*, int, int);

int GetBitPerPixel (DEV_T FAR*);

